-------------------------------------------------------------------------------
-- $Id: rx_state_2.vhd,v 1.1 2007/12/11 06:41:13 sanjayk Exp $
-------------------------------------------------------------------------------
-- rx_state - entity/architecture pair
-------------------------------------------------------------------------------
--  ***************************************************************************
--  **  Copyright(C) 2006 by Xilinx, Inc. All rights reserved.               **
--  **                                                                       **
--  **  This text contains proprietary, confidential                         **
--  **  information of Xilinx, Inc. , is distributed by                      **
--  **  under license from Xilinx, Inc., and may be used,                    **
--  **  copied and/or disclosed only pursuant to the terms                   **
--  **  of a valid license agreement with Xilinx, Inc.                       **
--  **                                                                       **
--  **  Unmodified source code is guaranteed to place and route,             **
--  **  function and run at speed according to the datasheet                 **
--  **  specification. Source code is provided "as-is", with no              **
--  **  obligation on the part of Xilinx to provide support.                 **
--  **                                                                       **
--  **  Xilinx Hotline support of source code IP shall only include          **
--  **  standard level Xilinx Hotline support, and will only address         **
--  **  issues and questions related to the standard released Netlist        **
--  **  version of the core (and thus indirectly, the original core source). **
--  **                                                                       **
--  **  The Xilinx Support Hotline does not have access to source            **
--  **  code and therefore cannot answer specific questions related          **
--  **  to source HDL. The Xilinx Support Hotline will only be able          **
--  **  to confirm the problem in the Netlist version of the core.           **
--  **                                                                       **
--  **  This copyright and support notice must be retained as part           **
--  **  of this text at all times.                                           **
--  ***************************************************************************
--
-------------------------------------------------------------------------------
-- Filename:        rx_state.vhd
-- 
-- Description:     
--                  This file contains the receive control state machine.
-- VHDL-Standard:   VHDL'93
-------------------------------------------------------------------------------
-- Structure:   
--
--  xps_ethernetlite.vhd
--      \
--      \-- xemac.vhd
--           \
--           \--xps_ipif_ssp1.vhd
--           \
--           \-- emac_dpram.vhd                     
--           \    \                     
--           \    \-- RAMB16_S36_S36
--           \    \     or
--           \    \-- RAMB4_S8_S8
--           \                          
--           \-- emac.vhd                     
--                \                     
--                \                     
--                \-- receive.vhd
--                \      rx_state_2.vhd
--                \      rx_intrfce.vhd
--                \         veasyncfifo5w15ddstrb.edn
--                \            or
--                \         v2asyncfifo5w15ddstrb.edn
--                \      cntr4bit.vhd
--                \      nibshiftin.vhd
--                \      nibshiftdain.vhd
--                \      cntr11bit.vhd
--                \      cntr12bit.vhd
--                \      dacheck.vhd
--                \      rxlengthcntr.vhd
--                \      crcgenrx.vhd
--                \      rxcrcnibcnt.vhd
--                \                     
--                \-- transmit_2.vhd
--                       crcgentx.vhd
--                          crcnibshiftreg
--                       tx_intrfce.vhd
--                          veasyncfifo5w15ddstrb.edn
--                             or
--                          v2asyncfifo5w15ddstrb.edn
--                       cntr11bit.vhd
--                       nibshiftout.vhd
--                       tx_state_3.vhd
--                       deferral.vhd
--                          cntr5bit.vhd
--                          defer_state.vhd
--                       bocntr.vhd
--                          lfsr16.vhd
--                       msh_cnt.vhd
--                       ld_arith_reg.vhd
--
-------------------------------------------------------------------------------
-- Author:   MSH
-- History:    
-- RSK       09/27/2006     First Version
-- ^^^^^^
--           First version of xps_ethernetlite, Based on opb_ethernetlite v1.01.b 
-- ~~~~~~
-- RSK       12/14/2006
-- ^^^^^^
--           The states of the FSM are encoded to a signal rd_destAddrNib
--           The output mac_addr_ram_addr_rd is registered
--           Added a case statement for the signal mac_addr_ram_addr_rd_D
-- ~~~~~~
-------------------------------------------------------------------------------
-- Naming Conventions:
--      active low signals:                     "*_n"
--      clock signals:                          "clk", "clk_div#", "clk_#x" 
--      reset signals:                          "rst", "rst_n" 
--      generics:                               "C_*" 
--      user defined types:                     "*_TYPE" 
--      state machine next state:               "*_ns" 
--      state machine current state:            "*_cs" 
--      combinatorial signals:                  "*_com" 
--      pipelined or register delay signals:    "*_d#" 
--      counter signals:                        "*cnt*"
--      clock enable signals:                   "*_ce" 
--      internal version of output port         "*_i"
--      device pins:                            "*_pin" 
--      ports:                                  - Names begin with Uppercase 
--      processes:                              "*_PROCESS" 
--      component instantiations:               "<ENTITY_>I_<#|FUNC>
-------------------------------------------------------------------------------
-- 
library ieee;
use ieee.STD_LOGIC_1164.all;
use ieee.numeric_std.UNSIGNED;
use ieee.numeric_std."+";

--Package file that contains constant definition for RESET_ACTIVE
library xps_ethernetlite_v2_00_c;
use xps_ethernetlite_v2_00_c.all;
-- synopsys translate_off
Library XilinxCoreLib;
library simprim;
-- synopsys translate_on
library unisim;
use unisim.Vcomponents.all;
-------------------------------------------------------------------------------
-- Port Declaration
-------------------------------------------------------------------------------
entity rx_state is
  generic (
    C_DUPLEX : integer := 1
      -- 1 = full duplex, 0 = half duplex    
    );
  port (
        Emac_rx_rd_data_d1     : in std_logic_vector(0 to 5);   -- 03-26-04
        Receive_enable         : out std_logic;   -- 03-26-04
        RxBusFifoRdAck         : in std_logic;  
        BusFifoEmpty           : in std_logic;
        Collision              : in std_logic;
        DataValid              : in std_logic;
        RxError                : in std_logic;
        BusFifoData            : in std_logic_vector(0 to 3);
        CrcOk                  : in std_logic;
        BusFifoRd              : out std_logic;
        RxAbortRst             : out std_logic;
        RxCrcRst               : out std_logic;
        RxCrcEn                : out std_logic;   
        Rx_addr_load           : out std_logic;
        Rx_addr_en             : out std_logic;
        Rx_start               : out std_logic;
        Rx_done                : out std_logic;
        Rx_done_irq            : out std_logic;
        Rx_pong_ping_l         : in  std_logic;
        Rx_DPM_ce              : out std_logic;
        Rx_DPM_wr_data         : out std_logic_vector (0 to 3);
        Rx_DPM_rd_data         : in std_logic_vector (0 to 3);    
        Rx_DPM_wr_rd_n         : out std_logic;
        Rx_idle                : out std_logic;

        Mac_addr_ram_addr_rd   : out std_logic_vector(0 to 3);
        Mac_addr_ram_data      : in std_logic_vector (0 to 3);
    
        Clk                    : in std_logic;
        Rst                    : in std_logic
        ); 
end rx_state;


architecture imp of rx_state is

type bo2sl_type is array (boolean) of std_logic;
constant bo2sl : bo2sl_type := (false => '0', true => '1');
 
signal rState_i                       :std_logic_vector (0 to 22);  
			      
signal idle                           :std_logic; -- state 0
signal waitForSfd1                    :std_logic; -- state 1
signal sfd1CheckBusFifoEmpty          :std_logic; -- state 2
signal waitForSfd2                    :std_logic; -- state 3
signal startReadDestAdrNib            :std_logic; -- state 4

signal readDestAdrNib1                :std_logic; -- state 5
signal readDestAdrNib2                :std_logic; -- state 6
signal readDestAdrNib3                :std_logic; -- state 7
signal readDestAdrNib4                :std_logic; -- state 8
signal readDestAdrNib5                :std_logic; -- state 9
signal readDestAdrNib6                :std_logic; -- state 10
signal readDestAdrNib7                :std_logic; -- state 11
signal readDestAdrNib8                :std_logic; -- state 12
signal readDestAdrNib9                :std_logic; -- state 13
signal readDestAdrNib10               :std_logic; -- state 14
signal readDestAdrNib11               :std_logic; -- state 15
signal readDestAdrNib12               :std_logic; -- state 16
			             
signal rdDestAddrNib                  :std_logic_vector(0 to 3);
signal rdDestAddrNib_D                :std_logic_vector(0 to 3);
signal rdDestAddrNib_eq_0             :std_logic;
signal rdDestAddrNib_eq_1             :std_logic;
signal rdDestAddrNib_eq_12            :std_logic;
			             
signal startReadDataNib               :std_logic; -- state 17
signal crcCheck                       :std_logic; -- state 18
signal writeFinalData                 :std_logic; -- state 19
signal rxDone                         :std_logic; -- state 20
signal receiveRst                     :std_logic; -- state 21
signal rxCollision                    :std_logic; -- state 22
			             
signal idle_D                         :std_logic; -- state 0
signal waitForSfd1_D                  :std_logic; -- state 1
signal sfd1CheckBusFifoEmpty_D        :std_logic; -- state 2
signal waitForSfd2_D                  :std_logic; -- state 3
signal startReadDestAdrNib_D          :std_logic; -- state 4

signal readDestAdrNib1_D               :std_logic; -- state 5
signal readDestAdrNib2_D               :std_logic; -- state 6
signal readDestAdrNib3_D               :std_logic; -- state 7
signal readDestAdrNib4_D               :std_logic; -- state 8
signal readDestAdrNib5_D               :std_logic; -- state 9
signal readDestAdrNib6_D               :std_logic; -- state 10
signal readDestAdrNib7_D               :std_logic; -- state 11
signal readDestAdrNib8_D               :std_logic; -- state 12
signal readDestAdrNib9_D               :std_logic; -- state 13
signal readDestAdrNib10_D              :std_logic; -- state 14
signal readDestAdrNib11_D              :std_logic; -- state 15
signal readDestAdrNib12_D              :std_logic; -- state 16
			             
signal startReadDataNib_D              :std_logic; -- state 17
signal crcCheck_D                      :std_logic; -- state 18
signal writeFinalData_D                :std_logic; -- state 19
signal rxDone_D                        :std_logic; -- state 20
signal receiveRst_D                    :std_logic; -- state 21
signal rxCollision_D                   :std_logic; -- state 22

signal goto_idle_1                     :std_logic; -- state 0
signal goto_idle_2                     :std_logic; -- state 0
signal goto_idle_3                     :std_logic; -- state 0
signal goto_idle_4                     :std_logic; -- state 0
signal goto_waitForSfd1                :std_logic; -- state 1
signal goto_sfd1CheckBusFifoEmpty_1    :std_logic; -- state 2
signal goto_sfd1CheckBusFifoEmpty_2    :std_logic; -- state 2
signal goto_waitForSfd2                :std_logic; -- state 3
signal goto_startReadDestAdrNib_1      :std_logic; -- state 4

signal goto_readDestAdrNib1            :std_logic; -- state 5
signal goto_readDestAdrNib2            :std_logic; -- state 6
signal goto_readDestAdrNib3            :std_logic; -- state 7
signal goto_readDestAdrNib4            :std_logic; -- state 8
signal goto_readDestAdrNib5            :std_logic; -- state 9
signal goto_readDestAdrNib6            :std_logic; -- state 10
signal goto_readDestAdrNib7            :std_logic; -- state 11
signal goto_readDestAdrNib8            :std_logic; -- state 12
signal goto_readDestAdrNib9            :std_logic; -- state 13
signal goto_readDestAdrNib10           :std_logic; -- state 14
signal goto_readDestAdrNib11           :std_logic; -- state 15
signal goto_readDestAdrNib12           :std_logic; -- state 16

signal goto_startReadDataNib_2         :std_logic; -- state 17
signal goto_crcCheck                   :std_logic; -- state 18
signal goto_writeFinalData             :std_logic; -- state 19
signal goto_rxDone_3                   :std_logic; -- state 20
signal goto_receiveRst_1               :std_logic; -- state 21
signal goto_receiveRst_2               :std_logic; -- state 21
signal goto_receiveRst_3               :std_logic; -- state 21
signal goto_receiveRst_5               :std_logic; -- state 21
signal goto_receiveRst_9               :std_logic; -- state 21
signal goto_receiveRst_10              :std_logic; -- state 21
signal goto_receiveRst_14              :std_logic; -- state 21
signal goto_rxCollision_1              :std_logic; -- state 22
signal goto_rxCollision_2              :std_logic; -- state 22
signal goto_rxCollision_5              :std_logic; -- state 22

signal stay_idle                       :std_logic; -- state 0
signal stay_sfd1CheckBusFifoEmpty      :std_logic; -- state 2
signal stay_startReadDestAdrNib        :std_logic; -- state 4

signal stay_readDestAdrNib1            :std_logic; -- state 5
signal stay_readDestAdrNib2            :std_logic; -- state 6
signal stay_readDestAdrNib3            :std_logic; -- state 7
signal stay_readDestAdrNib4            :std_logic; -- state 8
signal stay_readDestAdrNib5            :std_logic; -- state 9
signal stay_readDestAdrNib6            :std_logic; -- state 10
signal stay_readDestAdrNib7            :std_logic; -- state 11
signal stay_readDestAdrNib8            :std_logic; -- state 12
signal stay_readDestAdrNib9            :std_logic; -- state 13
signal stay_readDestAdrNib10           :std_logic; -- state 14
signal stay_readDestAdrNib11           :std_logic; -- state 15
signal stay_readDestAdrNib12           :std_logic; -- state 16

signal stay_startReadDataNib           :std_logic; -- state 17

signal state_machine_rst               : std_logic;
signal full_half_n                     : std_logic;
signal checkingBroadcastAdr_i          : std_logic;
signal checkingBroadcastAdr_reg        : std_logic;
signal busFifoData_is_5                : std_logic;
signal busFifoData_is_13               : std_logic;
signal busFifoData_not_5               : std_logic;
signal busFifoData_not_13              : std_logic;
signal bcastAddrGood                   : std_logic;
signal ucastAddrGood                   : std_logic;

signal rxie                            : std_logic; --rx interrupt enable
signal rxie_ce                         : std_logic; --only load on ping memory
signal crcokr1                         : std_logic;
signal crcokin                         : std_logic;
signal rxcrcen_i                       : std_logic;
signal mac_addr_ram_addr_rd_D          : std_logic_vector(0 to 3);
signal readDestAdrNib1to12             : std_logic_vector(1 to 12);
signal readDestAdrNib1to12_D           : std_logic_vector(1 to 12);
signal readDestAdrNib1to12_D0          : std_logic_vector(1 to 5);
signal readDestAdrNib1to12_D1          : std_logic_vector(1 to 6);
signal readDestAdrNib1to12_D2          : std_logic_vector(1 to 6);
signal readDestAdrNib1to12_D3          : std_logic_vector(1 to 6);
--ravi
signal rdDestAddrNib_D_t           : std_logic_vector(0 to 3);
signal rdDestAddrNib_D_t_Q         : std_logic_vector(0 to 3);

signal goto_readDestAdrNib1_t :std_logic;
signal stay_readDestAdrNib1_t :std_logic;

signal goto_readDestAdrNib2_t :std_logic;
signal stay_readDestAdrNib2_t :std_logic;
signal goto_readDestAdrNib3_t :std_logic;
signal stay_readDestAdrNib3_t :std_logic;
signal goto_readDestAdrNib4_t :std_logic;
signal stay_readDestAdrNib4_t :std_logic;
signal goto_readDestAdrNib5_t :std_logic;
signal stay_readDestAdrNib5_t :std_logic;
signal goto_readDestAdrNib6_t :std_logic;
signal stay_readDestAdrNib6_t :std_logic;
signal goto_readDestAdrNib7_t :std_logic;
signal stay_readDestAdrNib7_t :std_logic;
signal goto_readDestAdrNib8_t :std_logic;
signal stay_readDestAdrNib8_t :std_logic;
signal goto_readDestAdrNib9_t :std_logic;
signal stay_readDestAdrNib9_t :std_logic;

signal goto_readDestAdrNib10_t :std_logic;
signal stay_readDestAdrNib10_t :std_logic;
signal goto_readDestAdrNib11_t :std_logic;
signal stay_readDestAdrNib11_t :std_logic;
signal goto_readDestAdrNib12_t :std_logic;
signal stay_readDestAdrNib12_t :std_logic;


component FDR
  port (
    Q : out std_logic;
    C : in std_logic;
    D : in std_logic;
    R : in std_logic
  );
end component;

component FDS
  port (
    Q : out std_logic;
    C : in std_logic;
    D : in std_logic;
    S : in std_logic
  );
end component;

component FDRE
  port (
    Q : out std_logic;
    C : in std_logic;
    CE : in std_logic;
    D : in std_logic;
    R : in std_logic
  );
end component;


begin
  
crcokin <= ((CrcOk      -- set
           or crcokr1)  -- keep
           and (not(rxcrcen_i) or CrcOk)); -- clear when 0

crcokdelay: FDR
  port map (
    Q => crcokr1, --[out]
    C => Clk, --[in]
    D => crcokin, --[in]
    R => Rst --[in]
  );
  
  Rx_done     <= rxDone; -- added Rx_done output for ping pong control
  Rx_done_irq <= rxDone and rxie; -- added Rx_done_irq output for interrupt
  
  busFifoData_is_5         <= not(BusFifoData(0)) and BusFifoData(1) and 
    not(BusFifoData(2)) and BusFifoData(3);
  busFifoData_not_5        <= not(busFifoData_is_5);
  busFifoData_is_13        <=  BusFifoData(0) and BusFifoData(1) and 
    not(BusFifoData(2)) and BusFifoData(3);
  busFifoData_not_13       <= not(busFifoData_is_13);
  
  
  state_machine_rst <= Rst;

  --ToDo, remove or adjust for rdDestAddrNib when readdDestAddrNib<x> are removed.
--  rState_i <= idle & waitForSfd1 & sfd1CheckBusFifoEmpty & waitForSfd2 & 
--    startReadDestAdrNib & readDestAdrNib1 & readDestAdrNib2 & readDestAdrNib3
--    & readDestAdrNib4 & readDestAdrNib5 & readDestAdrNib6 & readDestAdrNib7 & 
--    readDestAdrNib8 & readDestAdrNib9 & readDestAdrNib10 & readDestAdrNib11 & 
--    readDestAdrNib12& startReadDataNib & crcCheck & writeFinalData & rxDone
--    & receiveRst  & rxCollision  ;


-------------------------------------------------------------------------------
-- Recieve Interrupt Enable Register
-------------------------------------------------------------------------------
RXIE_I: FDRE
  port map (
    Q => rxie              , --[out]
    C => Clk               , --[in]
    CE => rxie_ce          , --[in]
    D => Rx_DPM_rd_data(0) , --[in]
    R => Rst                 --[in]
  );

   rxie_ce <= idle and not Rx_pong_ping_l;
   
-------------------------------------------------------------------------------
-- idle state
-------------------------------------------------------------------------------    
  goto_idle_1 <= rxDone;
  goto_idle_2 <= receiveRst;  
  goto_idle_3 <= waitForSfd1 and (not(DataValid) or busFifoData_not_5);
  goto_idle_4 <= waitForSfd2 and (not(DataValid) or 
    (busFifoData_not_5 and busFifoData_not_13));      
    
  stay_idle <= idle and not(goto_waitForSfd1); 
  
  idle_D <= goto_idle_1 or goto_idle_2 or goto_idle_3 or goto_idle_4 
    or stay_idle;

state0a: FDS
  port map (
    Q => idle,             --[out]
    C => Clk,              --[in]
    D => idle_D,           --[in]
    S => state_machine_rst --[in]
  );
-------------------------------------------------------------------------------
-- waitForSfd1 state
-------------------------------------------------------------------------------      
  goto_waitForSfd1 <= idle and (RxBusFifoRdAck or not(BusFifoEmpty)) and
    not(Rx_DPM_rd_data(3));
    
  waitForSfd1_D <= goto_waitForSfd1;

state1a: FDR
  port map (
    Q => waitForSfd1,      --[out]
    C => Clk,              --[in]
    D => waitForSfd1_D,    --[in]
    R => state_machine_rst --[in]
  );    
Rx_idle <= idle or waitForSfd1;  
-------------------------------------------------------------------------------
-- sfd1CheckBusFifoEmpty state
-------------------------------------------------------------------------------      
  goto_sfd1CheckBusFifoEmpty_1 <= waitForSfd1 and busFifoData_is_5 and DataValid;
  goto_sfd1CheckBusFifoEmpty_2 <= waitForSfd2 and busFifoData_is_5 and DataValid;
  
  stay_sfd1CheckBusFifoEmpty <= sfd1CheckBusFifoEmpty and not(goto_rxCollision_1)
    and not(goto_receiveRst_1) and not(goto_waitForSfd2);
    
  sfd1CheckBusFifoEmpty_D <= goto_sfd1CheckBusFifoEmpty_1 or 
    goto_sfd1CheckBusFifoEmpty_2 or stay_sfd1CheckBusFifoEmpty;

state2a: FDR
  port map (
    Q => sfd1CheckBusFifoEmpty,    --[out]
    C => Clk,                      --[in]
    D => sfd1CheckBusFifoEmpty_D,  --[in]
    R => state_machine_rst         --[in]
  ); 
-------------------------------------------------------------------------------
-- waitForSfd2 state
-------------------------------------------------------------------------------      
  goto_waitForSfd2 <= sfd1CheckBusFifoEmpty and not(goto_rxCollision_1) and 
    not(goto_receiveRst_1) and (RxBusFifoRdAck or not(BusFifoEmpty));
  
  waitForSfd2_D <= goto_waitForSfd2;

state3a: FDR
  port map (
    Q => waitForSfd2,      --[out]
    C => Clk,              --[in]
    D => waitForSfd2_D,    --[in]
    R => state_machine_rst --[in]
  );  
-------------------------------------------------------------------------------
--startReadDestAdrNib state
-------------------------------------------------------------------------------      
  goto_startReadDestAdrNib_1 <= waitForSfd2 and busFifoData_is_13 and DataValid;
  
  stay_startReadDestAdrNib <= startReadDestAdrNib and not(goto_rxCollision_2)
    and not(goto_receiveRst_2) and not(goto_readDestAdrNib1);
    
  startReadDestAdrNib_D <= goto_startReadDestAdrNib_1 or 
    stay_startReadDestAdrNib;

state4a: FDR
  port map (
    Q => startReadDestAdrNib,    --[out]
    C => Clk,                    --[in]
    D => startReadDestAdrNib_D,  --[in]
    R => state_machine_rst       --[in]
  );
-------------------------------------------------------------------------------
--readDestAdrNib1 state
-------------------------------------------------------------------------------      
  goto_readDestAdrNib1 <= startReadDestAdrNib and not(goto_rxCollision_2) and 
    not(goto_receiveRst_2) and RxBusFifoRdAck;
    
  stay_readDestAdrNib1 <= readDestAdrNib1 and not(RxBusFifoRdAck) and 
    not(goto_receiveRst_5) and not(goto_receiveRst_3); -- 05-20-04 lock-up fix?
  
  readDestAdrNib1_D <= goto_readDestAdrNib1 or stay_readDestAdrNib1;

state5a: FDR
  port map (
    Q => readDestAdrNib1,    --[out]
    C => Clk,                --[in]
    D => readDestAdrNib1_D,  --[in]
    R => state_machine_rst   --[in]
  );  

readDestAdrNib1to12_D <= readDestAdrNib1_D & readDestAdrNib2_D & readDestAdrNib3_D &
                       readDestAdrNib4_D & readDestAdrNib5_D & readDestAdrNib6_D &
                       readDestAdrNib7_D & readDestAdrNib8_D & readDestAdrNib9_D &
                       readDestAdrNib10_D & readDestAdrNib11_D & readDestAdrNib12_D;

-- readDestAdrNib1to12_D is mapped to rdDestAddrNib_D as following 
-- to work on the critical paths
--
--          when "100000000000" => rdDestAddrNib_D <= "0001";
--          when "010000000000" => rdDestAddrNib_D <= "0010";
--          when "001000000000" => rdDestAddrNib_D <= "0011";
--          when "000100000000" => rdDestAddrNib_D <= "0100";
--          when "000010000000" => rdDestAddrNib_D <= "0101";
--          when "000001000000" => rdDestAddrNib_D <= "0110";
--          when "000000100000" => rdDestAddrNib_D <= "0111";
--          when "000000010000" => rdDestAddrNib_D <= "1000";
--          when "000000001000" => rdDestAddrNib_D <= "1001";
--          when "000000000100" => rdDestAddrNib_D <= "1010";
--          when "000000000010" => rdDestAddrNib_D <= "1011";
--          when "000000000001" => rdDestAddrNib_D <= "1100";
--          when others         => rdDestAddrNib_D <= "0000";


rdDestAddrNib_D(0) <= readDestAdrNib1to12_D(8)  or 
                      readDestAdrNib1to12_D(9)  or
                      readDestAdrNib1to12_D(10) or
                      readDestAdrNib1to12_D(11) or
                      readDestAdrNib1to12_D(12);

rdDestAddrNib_D(1) <= readDestAdrNib1to12_D(4) or 
                      readDestAdrNib1to12_D(5) or
                      readDestAdrNib1to12_D(6) or
                      readDestAdrNib1to12_D(7) or
                      readDestAdrNib1to12_D(12);

rdDestAddrNib_D(2) <= readDestAdrNib1to12_D(2) or
                      readDestAdrNib1to12_D(3) or
                      readDestAdrNib1to12_D(6) or
                      readDestAdrNib1to12_D(7) or
                      readDestAdrNib1to12_D(10) or
                      readDestAdrNib1to12_D(11);

rdDestAddrNib_D(3) <= readDestAdrNib1to12_D(1) or
                      readDestAdrNib1to12_D(3) or
                      readDestAdrNib1to12_D(5) or
                      readDestAdrNib1to12_D(7) or
                      readDestAdrNib1to12_D(9) or
                      readDestAdrNib1to12_D(11);

process (Clk)
begin
   if Clk'event and Clk='1' then
      if state_machine_rst = '1' then
           rdDestAddrNib <= "0000";
      else
           rdDestAddrNib <= rdDestAddrNib_D;
      end if;
   end if;
end process;

  rdDestAddrNib_eq_0  <= bo2sl(rdDestAddrNib = "0000");
  rdDestAddrNib_eq_1  <= bo2sl(rdDestAddrNib = "0001");
  rdDestAddrNib_eq_12 <= bo2sl(rdDestAddrNib = "1011");

----
-- Receive state machine, Duplicate of readDestAdrNib1 to readDestAdrNib12
--

process (Clk)
begin
   if Clk'event and Clk='1' then
      if state_machine_rst = '1' then
           rdDestAddrNib_D_t_Q <= "0000";
      else
           rdDestAddrNib_D_t_Q <= rdDestAddrNib_D_t;
      end if;
   end if;
end process;

process (rdDestAddrNib,startReadDestAdrNib,goto_rxCollision_2,
         goto_receiveRst_2,RxBusFifoRdAck,goto_receiveRst_3,bcastAddrGood,
         ucastAddrGood,goto_receiveRst_5,goto_readDestAdrNib1_t,stay_readDestAdrNib1_t,
         rdDestAddrNib_D_t_q,stay_readDestAdrNib12_t)
begin
----

goto_readDestAdrNib1_t <= '0';
stay_readDestAdrNib1_t <= '0';
stay_readDestAdrNib2_t <= '0';
stay_readDestAdrNib3_t <= '0';
stay_readDestAdrNib4_t <= '0';
stay_readDestAdrNib5_t <= '0';
stay_readDestAdrNib6_t <= '0';
stay_readDestAdrNib7_t <= '0';
stay_readDestAdrNib8_t <= '0';
stay_readDestAdrNib9_t <= '0';
stay_readDestAdrNib10_t <= '0';
stay_readDestAdrNib11_t <= '0';
stay_readDestAdrNib12_t <= '0';


rdDestAddrNib_D_t <= rdDestAddrNib_D_t_q;
   case (rdDestAddrNib_D_t_q) is

     when "0000" => 
     
                    goto_readDestAdrNib1_t <= startReadDestAdrNib and not(goto_rxCollision_2) and 
                    not(goto_receiveRst_2) and RxBusFifoRdAck;

                    if goto_readDestAdrNib1_t = '1' then
                      rdDestAddrNib_D_t <= "0001";
                      end if;
                      

     when "0001" =>  if (RxBusFifoRdAck and (bcastAddrGood or ucastAddrGood) and
                        not(goto_receiveRst_5) and not(goto_receiveRst_3)) = '1' then
                        rdDestAddrNib_D_t <= "0010";
                      end if;

                      stay_readDestAdrNib1_t <= not(RxBusFifoRdAck) and not(goto_receiveRst_5) 
                      and not(goto_receiveRst_3);

                     if stay_readDestAdrNib1_t = '1' then
                      rdDestAddrNib_D_t <= "0001";
                      end if;
                      
                      

     when "0010" =>  if (RxBusFifoRdAck and (bcastAddrGood or ucastAddrGood) and 
                        not(goto_receiveRst_5) and not(goto_receiveRst_3)) = '1' then
                        rdDestAddrNib_D_t <= "0011";
                      end if;
                      

     when "0011" =>  if (RxBusFifoRdAck and (bcastAddrGood or ucastAddrGood) and 
                        not(goto_receiveRst_5) and not(goto_receiveRst_3)) = '1' then
                        rdDestAddrNib_D_t <= "0100";
                      end if;
                      

     when "0100" =>  if (RxBusFifoRdAck and (bcastAddrGood or ucastAddrGood) and 
                        not(goto_receiveRst_5) and not(goto_receiveRst_3)) = '1' then
                        rdDestAddrNib_D_t <= "0101";
                      end if;
                      
--                      stay_readDestAdrNib4_t <= not(RxBusFifoRdAck) and not(goto_receiveRst_5) 
--		      and not(goto_receiveRst_3);
--
--      		      if stay_readDestAdrNib4_t = '1' then
--                         rdDestAddrNib_D_t <= "0100";
--		       end if;                      
                      

     when "0101" =>  if (RxBusFifoRdAck and (bcastAddrGood or ucastAddrGood) and 
                        not(goto_receiveRst_5) and not(goto_receiveRst_3)) = '1' then
                        rdDestAddrNib_D_t <= "0110";
                      end if;
                      

     when "0110" =>  if (RxBusFifoRdAck and (bcastAddrGood or ucastAddrGood) and 
                        not(goto_receiveRst_5) and not(goto_receiveRst_3)) = '1' then
                        rdDestAddrNib_D_t <= "0111";
                      end if;
                      
     when "0111" =>  if (RxBusFifoRdAck and (bcastAddrGood or ucastAddrGood) and 
                        not(goto_receiveRst_5) and not(goto_receiveRst_3)) = '1' then
                        rdDestAddrNib_D_t <= "1000";
                      end if;
                      
     when "1000" =>  if (RxBusFifoRdAck and (bcastAddrGood or ucastAddrGood) and 
                        not(goto_receiveRst_5) and not(goto_receiveRst_3)) = '1' then
                        rdDestAddrNib_D_t <= "1001";
                      end if;

     when "1001" =>  if (RxBusFifoRdAck and (bcastAddrGood or ucastAddrGood) and 
                        not(goto_receiveRst_5) and not(goto_receiveRst_3)) = '1' then
                        rdDestAddrNib_D_t <= "1010";
                      end if;
                      
     when "1010" =>  if (RxBusFifoRdAck and (bcastAddrGood or ucastAddrGood) and 
                        not(goto_receiveRst_5) and not(goto_receiveRst_3)) = '1' then
                        rdDestAddrNib_D_t <= "1011";
                      end if;
                      
     when "1011" =>  if (RxBusFifoRdAck and (bcastAddrGood or ucastAddrGood) and 
                        not(goto_receiveRst_5) and not(goto_receiveRst_3)) = '1' then
                        rdDestAddrNib_D_t <= "1100";
                      end if;
                      
     when "1100" =>  if (RxBusFifoRdAck and (bcastAddrGood or ucastAddrGood) and 
                        not(goto_receiveRst_5) and not(goto_receiveRst_3)) = '1' then
                        rdDestAddrNib_D_t <= "0000";
                        else
                        rdDestAddrNib_D_t <= "1100";
                      end if;

     
     when others =>  null;
     end case;
 end process;

--------------------------------------------------------------------------------
----readDestAdrNib2 state
---------------------------------------------------------------------------------      
  goto_readDestAdrNib2 <= readDestAdrNib1 and RxBusFifoRdAck and
    (bcastAddrGood or ucastAddrGood) and not(goto_receiveRst_5) and 
    not(goto_receiveRst_3);
  
  stay_readDestAdrNib2 <= readDestAdrNib2 and not(RxBusFifoRdAck) and 
    not(goto_receiveRst_5) and not(goto_receiveRst_3);
  
  readDestAdrNib2_D <= goto_readDestAdrNib2 or stay_readDestAdrNib2;

state6a: FDR
  port map (
    Q => readDestAdrNib2,    --[out]
    C => Clk,                --[in]
    D => readDestAdrNib2_D,  --[in]
    R => state_machine_rst   --[in]
  );    
-------------------------------------------------------------------------------
--readDestAdrNib3 state
-------------------------------------------------------------------------------      
  goto_readDestAdrNib3 <= readDestAdrNib2 and RxBusFifoRdAck and
    (bcastAddrGood or ucastAddrGood) and not(goto_receiveRst_5) and 
    not(goto_receiveRst_3);
 
  stay_readDestAdrNib3 <= readDestAdrNib3 and not(RxBusFifoRdAck) and 
    not(goto_receiveRst_5) and not(goto_receiveRst_3);
  
  readDestAdrNib3_D <= goto_readDestAdrNib3 or stay_readDestAdrNib3;

state7a: FDR
  port map (
    Q => readDestAdrNib3,    --[out]
    C => Clk,                --[in]
    D => readDestAdrNib3_D,  --[in]
    R => state_machine_rst   --[in]
  ); 
-----------------------------------------------------------------------------
--readDestAdrNib4 state
-----------------------------------------------------------------------------      
  goto_readDestAdrNib4 <= readDestAdrNib3 and RxBusFifoRdAck and
    (bcastAddrGood or ucastAddrGood) and not(goto_receiveRst_5) and 
    not(goto_receiveRst_3);
  
  stay_readDestAdrNib4 <= readDestAdrNib4 and not(RxBusFifoRdAck) and 
    not(goto_receiveRst_5) and not(goto_receiveRst_3);
  
  readDestAdrNib4_D <= goto_readDestAdrNib4 or stay_readDestAdrNib4;

state8a: FDR
  port map (
    Q => readDestAdrNib4,    --[out]
    C => Clk,                --[in]
    D => readDestAdrNib4_D,  --[in]
    R => state_machine_rst   --[in]
  ); 
-------------------------------------------------------------------------------
--readDestAdrNib5 state
-------------------------------------------------------------------------------      
  goto_readDestAdrNib5 <= readDestAdrNib4 and RxBusFifoRdAck and
    (bcastAddrGood or ucastAddrGood) and not(goto_receiveRst_5) and 
    not(goto_receiveRst_3);
  
  stay_readDestAdrNib5 <= readDestAdrNib5 and not(RxBusFifoRdAck) and 
    not(goto_receiveRst_5) and not(goto_receiveRst_3);
  
  readDestAdrNib5_D <= goto_readDestAdrNib5 or stay_readDestAdrNib5;

state9a: FDR
  port map (
    Q => readDestAdrNib5,    --[out]
    C => Clk,                --[in]
    D => readDestAdrNib5_D,  --[in]
    R => state_machine_rst   --[in]
  );
-------------------------------------------------------------------------------
--readDestAdrNib6 state
-------------------------------------------------------------------------------      
  goto_readDestAdrNib6 <= readDestAdrNib5 and RxBusFifoRdAck and
    (bcastAddrGood or ucastAddrGood) and not(goto_receiveRst_5) and 
    not(goto_receiveRst_3);  
  
  stay_readDestAdrNib6 <= readDestAdrNib6 and not(RxBusFifoRdAck) and 
    not(goto_receiveRst_5) and not(goto_receiveRst_3);
  
  readDestAdrNib6_D <= goto_readDestAdrNib6 or stay_readDestAdrNib6;

state10a: FDR
  port map (
    Q => readDestAdrNib6,    --[out]
    C => Clk,                --[in]
    D => readDestAdrNib6_D,  --[in]
    R => state_machine_rst   --[in]
  ); 
-------------------------------------------------------------------------------
--readDestAdrNib7 state
-------------------------------------------------------------------------------      
  goto_readDestAdrNib7 <= readDestAdrNib6 and RxBusFifoRdAck and
    (bcastAddrGood or ucastAddrGood) and not(goto_receiveRst_5) and 
    not(goto_receiveRst_3);  
  
  stay_readDestAdrNib7 <= readDestAdrNib7 and not(RxBusFifoRdAck) and 
    not(goto_receiveRst_5) and not(goto_receiveRst_3);
  
  readDestAdrNib7_D <= goto_readDestAdrNib7 or stay_readDestAdrNib7;

state11a: FDR
  port map (
    Q => readDestAdrNib7,    --[out]
    C => Clk,                --[in]
    D => readDestAdrNib7_D,  --[in]
    R => state_machine_rst   --[in]
  ); 
-------------------------------------------------------------------------------
--readDestAdrNib8 state
-------------------------------------------------------------------------------      
  goto_readDestAdrNib8 <= readDestAdrNib7 and RxBusFifoRdAck and
    (bcastAddrGood or ucastAddrGood) and not(goto_receiveRst_5) and 
    not(goto_receiveRst_3);  
  
  stay_readDestAdrNib8 <= readDestAdrNib8 and not(RxBusFifoRdAck) and 
    not(goto_receiveRst_5) and not(goto_receiveRst_3);
  
  readDestAdrNib8_D <= goto_readDestAdrNib8 or stay_readDestAdrNib8;

state12a: FDR
  port map (
    Q => readDestAdrNib8,    --[out]
    C => Clk,                --[in]
    D => readDestAdrNib8_D,  --[in]
    R => state_machine_rst   --[in]
  ); 
-------------------------------------------------------------------------------
--readDestAdrNib9 state
-------------------------------------------------------------------------------      
  goto_readDestAdrNib9 <= readDestAdrNib8 and RxBusFifoRdAck and
    (bcastAddrGood or ucastAddrGood) and not(goto_receiveRst_5) and 
    not(goto_receiveRst_3);  
  
  stay_readDestAdrNib9 <= readDestAdrNib9 and not(RxBusFifoRdAck) and 
    not(goto_receiveRst_5) and not(goto_receiveRst_3);
  
  readDestAdrNib9_D <= goto_readDestAdrNib9 or stay_readDestAdrNib9;

state13a: FDR
  port map (
    Q => readDestAdrNib9,    --[out]
    C => Clk,                --[in]
    D => readDestAdrNib9_D,  --[in]
    R => state_machine_rst   --[in]
  ); 
-------------------------------------------------------------------------------
--readDestAdrNib10 state
-------------------------------------------------------------------------------      
  goto_readDestAdrNib10 <= readDestAdrNib9 and RxBusFifoRdAck and
    (bcastAddrGood or ucastAddrGood) and not(goto_receiveRst_5) and 
    not(goto_receiveRst_3);  
  
  stay_readDestAdrNib10 <= readDestAdrNib10 and not(RxBusFifoRdAck) and 
    not(goto_receiveRst_5) and not(goto_receiveRst_3);
  
  readDestAdrNib10_D <= goto_readDestAdrNib10 or stay_readDestAdrNib10;

state14a: FDR
  port map (
    Q => readDestAdrNib10,    --[out]
    C => Clk,                 --[in]
    D => readDestAdrNib10_D,  --[in]
    R => state_machine_rst    --[in]
  ); 
-------------------------------------------------------------------------------
--readDestAdrNib11 state
-------------------------------------------------------------------------------      
  goto_readDestAdrNib11 <= readDestAdrNib10 and RxBusFifoRdAck and
    (bcastAddrGood or ucastAddrGood) and not(goto_receiveRst_5) and 
    not(goto_receiveRst_3);  
  
  stay_readDestAdrNib11 <= readDestAdrNib11 and not(RxBusFifoRdAck) and 
    not(goto_receiveRst_5) and not(goto_receiveRst_3);
  
  readDestAdrNib11_D <= goto_readDestAdrNib11 or stay_readDestAdrNib11;

state15a: FDR
  port map (
    Q => readDestAdrNib11,    --[out]
    C => Clk,                 --[in]
    D => readDestAdrNib11_D,  --[in]
    R => state_machine_rst    --[in]
  ); 
-------------------------------------------------------------------------------
--readDestAdrNib12 state
-------------------------------------------------------------------------------      
  goto_readDestAdrNib12 <= readDestAdrNib11 and RxBusFifoRdAck and
    (bcastAddrGood or ucastAddrGood) and not(goto_receiveRst_5) and 
    not(goto_receiveRst_3); 
  
  stay_readDestAdrNib12 <= readDestAdrNib12 and not(RxBusFifoRdAck) and 
    not(goto_receiveRst_5) and not(goto_receiveRst_3);
  
  readDestAdrNib12_D <= goto_readDestAdrNib12 or stay_readDestAdrNib12;

state16a: FDR
  port map (
    Q => readDestAdrNib12,    --[out]
    C => Clk,                 --[in]
    D => readDestAdrNib12_D,  --[in]
    R => state_machine_rst    --[in]
  );   
-------------------------------------------------------------------------------
--startReadDataNib state
-------------------------------------------------------------------------------      
  goto_startReadDataNib_2 <= rdDestAddrNib_eq_12 and RxBusFifoRdAck and
    (bcastAddrGood or ucastAddrGood) and not(goto_receiveRst_5) and 
    not(goto_receiveRst_3);
  
  stay_startReadDataNib <= startReadDataNib and not(goto_rxCollision_5) 
    and not(goto_receiveRst_9) and DataValid;
    
  startReadDataNib_D <= goto_startReadDataNib_2
    or stay_startReadDataNib;

state17a: FDR
  port map (
    Q => startReadDataNib,    --[out]
    C => Clk,                 --[in]
    D => startReadDataNib_D,  --[in]
    R => state_machine_rst    --[in]
  );  
-------------------------------------------------------------------------------
--crcCheck state
-------------------------------------------------------------------------------      
  goto_crcCheck <= startReadDataNib and not(DataValid) ;
  
  goto_receiveRst_1 <= sfd1CheckBusFifoEmpty and not(goto_rxCollision_1) and RxError;
  goto_receiveRst_2 <= startReadDestAdrNib and not(goto_rxCollision_2) and RxError;
  goto_receiveRst_9 <= startReadDataNib and not(goto_rxCollision_5) and RxError;

  crcCheck_D <= goto_crcCheck or goto_receiveRst_1 or goto_receiveRst_2 or goto_receiveRst_9;

state18a: FDR
  port map (
    Q => crcCheck,         --[out]
    C => Clk,              --[in]
    D => crcCheck_D,       --[in]
    R => state_machine_rst --[in]
  );  
-------------------------------------------------------------------------------
--writeFinalData state
-------------------------------------------------------------------------------      
  goto_writeFinalData <= crcCheck and crcokr1;
   
  writeFinalData_D <= goto_writeFinalData ;

state19a: FDR
  port map (
    Q => writeFinalData,    --[out]
    C => Clk,               --[in]
    D => writeFinalData_D,  --[in]
    R => state_machine_rst  --[in]
  );  
-------------------------------------------------------------------------------
--rxDone state
-------------------------------------------------------------------------------      
  goto_rxDone_3 <= writeFinalData ;
    
  rxDone_D <= goto_rxDone_3;

state20a: FDR
  port map (
    Q => rxDone,           --[out]
    C => Clk,              --[in]
    D => rxDone_D,         --[in]
    R => state_machine_rst --[in]
  );  
-------------------------------------------------------------------------------
--rxCollision state
-------------------------------------------------------------------------------    
  full_half_n <= '1'when C_DUPLEX = 1 else
                 '0';
  goto_rxCollision_1 <= sfd1CheckBusFifoEmpty and Collision and not(full_half_n);
  goto_rxCollision_2 <= startReadDestAdrNib and Collision and not(full_half_n);
  goto_rxCollision_5 <= startReadDataNib and Collision and not(full_half_n);
    
  rxCollision_D <= goto_rxCollision_1 or goto_rxCollision_2 or 
    goto_rxCollision_5;

state21a: FDR
  port map (
    Q => rxCollision,       --[out]
    C => Clk,               --[in]
    D => rxCollision_D,     --[in]
    R => state_machine_rst  --[in]
  );  
-------------------------------------------------------------------------------
--receiveRst state
-------------------------------------------------------------------------------      
  goto_receiveRst_3 <= not rdDestAddrNib_eq_0 and not(DataValid);

  goto_receiveRst_5 <=     not rdDestAddrNib_eq_0
                       and not(BusFifoEmpty)
                       and not(bcastAddrGood or ucastAddrGood);

  goto_receiveRst_10<= crcCheck and not(crcOkr1);
  goto_receiveRst_14<= rxCollision;
    
  receiveRst_D <= goto_receiveRst_3 
    or goto_receiveRst_5 or 
    goto_receiveRst_10 or goto_receiveRst_14;

state22a: FDR
  port map (
    Q => receiveRst,        --[out]
    C => Clk,               --[in]
    D => receiveRst_D,      --[in]
    R => state_machine_rst  --[in]
  );  
-------------------------------------------------------------------------------   
-- end of states
-------------------------------------------------------------------------------   

  a : process (Clk)
  begin  --
    if (Clk'event and Clk = '1') then     -- rising clock edge
      if (Rst = '1') then
        checkingBroadcastAdr_reg <= '0';
      else
        checkingBroadcastAdr_reg <= checkingBroadcastAdr_i;          
      end if;
    end if;
  end process a;
  
  fsmd : process( DataValid,RxBusFifoRdAck,idle
                , startReadDestAdrNib,startReadDataNib, sfd1CheckBusFifoEmpty,rxDone
                , writeFinalData, receiveRst, crcCheck, waitForSfd2
                , Emac_rx_rd_data_d1, checkingBroadcastAdr_reg,
                rdDestAddrNib, rdDestAddrNib_eq_0,rdDestAddrNib_eq_1
                ) 
  begin

    If (idle = '1') then
      RxCrcRst           <=  '1';
    else
      RxCrcRst           <=  '0';
    end if;


    if (    (   startReadDestAdrNib
             or not rdDestAddrNib_eq_0
             or (startReadDataNib and DataValid)
            )
        and RxBusFifoRdAck
       ) = '1'
    then
      RxCrcEn     <=   '1';
      rxCrcEn_i   <=   '1';
    else
      RxCrcEn     <=   '0';
      rxCrcEn_i   <=   '0';
    end if; 


    If ((idle = '1') or
        (sfd1CheckBusFifoEmpty = '1') or 
        (not rdDestAddrNib_eq_0 = '1') or
        (rxDone = '1') or   -- 03-26-04
        (writeFinalData = '1') or  -- 03-26-04
        (startReadDestAdrNib = '1') or
        (startreadDataNib = '1')) and (RxBusFifoRdAck = '0')then
      BusFifoRd          <=   '1';
    else
      BusFifoRd          <=   '0';
    end if; 

    If (receiveRst = '1') then
      RxAbortRst <= '1';
    else
      RxAbortRst <= '0';
    end if;   
    
    If (writeFinalData = '1' or crcCheck = '1' or rxDone = '1' or idle = '1') then
      Rx_addr_load <= '1'; --enable address constants
    else
      Rx_addr_load <= '0';
    end if;    

    If RxBusFifoRdAck = '1' and
       (
        (startReadDestAdrNib = '1') or  -- 03-26-04
        (not rdDestAddrNib_eq_0 = '1') or
        (startReadDataNib = '1')
       ) then
      Rx_addr_en <= '1'; --enable address increment
    else
      Rx_addr_en <= '0';
    end if; 

    If (waitForSfd2 = '1')then
      Rx_start <= '1'; -- reset address to 0 for start of receive
    else
      Rx_start <= '0';
    end if;
    
    If (idle = '1') or
       (writeFinalData = '1') or
       (crcCheck = '1') or
       (rxDone = '1') or
      (
       (
        (startReadDestAdrNib = '1') or  -- 03-26-04
        (not rdDestAddrNib_eq_0 = '1') or
        (startReadDataNib = '1')
       ) and (RxBusFifoRdAck = '1')
      ) then
      Rx_DPM_ce <= '1';
    else
      Rx_DPM_ce <= '0';
    end if;
    

    if (writeFinalData = '1') or 
      (startReadDestAdrNib = '1') or  -- 03-26-04
      (not rdDestAddrNib_eq_0 = '1') or
      (startReadDataNib = '1') then
      Rx_DPM_wr_rd_n <= '1';   
    else   
      Rx_DPM_wr_rd_n <= '0';
    end if;

    if (idle = '1') then
      checkingBroadcastAdr_i <= '0';  -- reset
    elsif (readDestAdrNib1 = '1' and Emac_rx_rd_data_d1(0 to 3) = x"f") then   -- 06-09-04 Use delayed data for compare
      checkingBroadcastAdr_i <= '1'; -- set
    else
      checkingBroadcastAdr_i <= checkingBroadcastAdr_reg; -- stay the same
    end if;   

  end process fsmd;

  Rx_DPM_wr_data <= x"1" when writeFinalData = '1' and rxie = '0' else
                    x"9" when writeFinalData = '1' and rxie = '1' else
                    BusFifoData;

MARAR_PROC : process (rdDestAddrNib_D_t, idle_D, startReadDestAdrNib_D)
begin
    case rdDestAddrNib_D_t is
      when "0001" => mac_addr_ram_addr_rd_D <= x"0";
      when "0010" => mac_addr_ram_addr_rd_D <= x"1";
      when "0011" => mac_addr_ram_addr_rd_D <= x"2";
      when "0100" => mac_addr_ram_addr_rd_D <= x"3";
      when "0101" => mac_addr_ram_addr_rd_D <= x"4";
      when "0110" => mac_addr_ram_addr_rd_D <= x"5";
      when "0111" => mac_addr_ram_addr_rd_D <= x"6";
      when "1000" => mac_addr_ram_addr_rd_D <= x"7";
      when "1001" => mac_addr_ram_addr_rd_D <= x"8";
      when "1010" => mac_addr_ram_addr_rd_D <= x"9";
      when "1011" => mac_addr_ram_addr_rd_D <= x"a";
      when "1100" => mac_addr_ram_addr_rd_D <= x"b";
      when others => mac_addr_ram_addr_rd_D <= x"0";
    end case;
    if (idle_D or startReadDestAdrNib_D) = '1' then
        mac_addr_ram_addr_rd_D <= x"0";
    end if;
end process;

OUTPUT_REG:process (Clk)
begin
if Clk'event and Clk = '1' then
    if Rst = '1' then
      Mac_addr_ram_addr_rd <= (others => '0');
    else
      Mac_addr_ram_addr_rd <= mac_addr_ram_addr_rd_D;
    end if;
end if;
end process OUTPUT_REG;



bcastAddrGood <= '1' when checkingBroadcastAdr_i = '1' and Emac_rx_rd_data_d1(0 to 3) = x"F" else  -- 03-26-04
                 '0';   
--ucastAddrGood <= '1' when checkingBroadcastAdr_i = '0' and (Emac_rx_rd_data_d1(0 to 3) = Mac_addr_ram_data) else  -- 03-26-04
--                 '0';    
ucastAddrGood <= '1';    
                 
Receive_enable <= not(crcCheck or rxDone or writeFinalData or receiveRst);  -- 03-26-04

end imp;
