-------------------------------------------------------------------------------
-- $Id: xemac.vhd,v 1.1 2007/12/11 06:41:17 sanjayk Exp $
-------------------------------------------------------------------------------
-- xemac.vhd - entity/architecture pair
-------------------------------------------------------------------------------
--  ***************************************************************************
--  **  Copyright(C) 2006 by Xilinx, Inc. All rights reserved.               **
--  **                                                                       **
--  **  This text contains proprietary, confidential                         **
--  **  information of Xilinx, Inc. , is distributed by                      **
--  **  under license from Xilinx, Inc., and may be used,                    **
--  **  copied and/or disclosed only pursuant to the terms                   **
--  **  of a valid license agreement with Xilinx, Inc.                       **
--  **                                                                       **
--  **  Unmodified source code is guaranteed to place and route,             **
--  **  function and run at speed according to the datasheet                 **
--  **  specification. Source code is provided "as-is", with no              **
--  **  obligation on the part of Xilinx to provide support.                 **
--  **                                                                       **
--  **  Xilinx Hotline support of source code IP shall only include          **
--  **  standard level Xilinx Hotline support, and will only address         **
--  **  issues and questions related to the standard released Netlist        **
--  **  version of the core (and thus indirectly, the original core source). **
--  **                                                                       **
--  **  The Xilinx Support Hotline does not have access to source            **
--  **  code and therefore cannot answer specific questions related          **
--  **  to source HDL. The Xilinx Support Hotline will only be able          **
--  **  to confirm the problem in the Netlist version of the core.           **
--  **                                                                       **
--  **  This copyright and support notice must be retained as part           **
--  **  of this text at all times.                                           **
--  ***************************************************************************
--
-------------------------------------------------------------------------------
-- Filename:        xemac.vhd
--
-- Description:     Design file for the Ethernet Lite MAC with
--                  IPIF elements included. 
-- VHDL-Standard:   VHDL'93
-------------------------------------------------------------------------------
-- Structure:   
--
--  xps_ethernet.vhd
--      \
--      \-- xemac.vhd
--           \
--           \-- xps_ipif_ssp1.vhd
--           \    
--           \
--           \-- emac_dpram.vhd                     
--           \    \                     
--           \    \-- RAMB16_S36_S36
--           \    \     or
--           \    \-- RAMB4_S8_S8
--           \                          
--           \-- emac.vhd                     
--                \                     
--                \-- receive.vhd
--                \      rx_state_2.vhd
--                \      rx_intrfce.vhd
--                \         veasyncfifo5w15ddstrb.edn
--                \            or
--                \         veasyncfifo5w15dbmem.edn
--                \            or
--                \         v2asyncfifo5w15ddstrb.edn
--                \            or
--                \         v2asyncfifo5w15dbmem.edn
--                \      cntr4bit.vhd
--                \      nibshiftin.vhd
--                \      nibshiftdain.vhd
--                \      cntr11bit.vhd
--                \      cntr12bit.vhd
--                \      dacheck.vhd
--                \      rxlengthcntr.vhd
--                \      crcgenrx.vhd
--                \      rxcrcnibcnt.vhd
--                \                     
--                \-- transmit_2.vhd
--                       crcgentx.vhd
--                          crcnibshiftreg
--                       tx_intrfce.vhd
--                          veasyncfifo5w15ddstrb.edn
--                             or
--                          veasyncfifo5w15dbmem.edn
--                             or
--                          v2asyncfifo5w15ddstrb.edn
--                             or
--                          v2asyncfifo5w15dbmem.edn
--                       cntr11bit.vhd
--                       nibshiftout.vhd
--                       tx_state_3.vhd
--                       deferral.vhd
--                          cntr5bit.vhd
--                          defer_state.vhd
--                       bocntr.vhd
--                          lfsr16.vhd
--                       msh_cnt.vhd
--                       ld_arith_reg.vhd
--
-------------------------------------------------------------------------------
-- Author:         MSH
-- History:
--  RSK            11/06/06
-- ^^^^^^
--                 First Version, Based on opb_ethernetlite v1.01.b
--  ~~~~~
--  SK             12/06/07 updated version and removed license attribute
-- ^^^^^^
--                 Updated Version, Based on xps_ethernetlite v1.00.a
--                 Removed the attribute "check_license" to make the core free core
--  ~~~~~
-------------------------------------------------------------------------------
-- Naming Conventions:
--      active low signals:                     "*_n"
--      clock signals:                          "clk", "clk_div#", "clk_#x" 
--      reset signals:                          "rst", "rst_n" 
--      generics:                               "C_*" 
--      user defined types:                     "*_TYPE" 
--      state machine next state:               "*_ns" 
--      state machine current state:            "*_cs" 
--      combinatorial signals:                  "*_com" 
--      pipelined or register delay signals:    "*_d#" 
--      counter signals:                        "*cnt*"
--      clock enable signals:                   "*_ce" 
--      internal version of output port         "*_i"
--      device pins:                            "*_pin" 
--      ports:                                  - Names begin with Uppercase 
--      processes:                              "*_PROCESS" 
--      component instantiations:               "<ENTITY_>I_<#|FUNC>
-------------------------------------------------------------------------------
library ieee;
use ieee.STD_LOGIC_1164.all;
library ieee;
use ieee.std_logic_arith.all;
--Package file that contains constant definition for RESET_ACTIVE
library xps_ethernetlite_v2_00_c;
use xps_ethernetlite_v2_00_c.all;

library proc_common_v2_00_a;
use proc_common_v2_00_a.all;
use proc_common_v2_00_a.family.all;
library unisim;
use unisim.Vcomponents.all;
-------------------------------------------------------------------------------
-- Port Declaration
-------------------------------------------------------------------------------

entity xemac is
  generic (
    C_DUPLEX              : integer := 1; -- 1 = full duplex, 0 = half duplex
    C_RX_PING_PONG        : integer := 0; -- 1 = ping-pong memory used for 
                                          --     receive buffer
    C_TX_PING_PONG        : integer := 0; -- 1 = ping-pong memory used for 
                                          --     transmit buffer
    NODE_MAC              : bit_vector := x"00005e00FACE"; 
                                          --  power up defaul MAC address
    C_BASEADDR            : std_logic_vector := X"60000000";
                                          --  Assigned Base Address for
                                          --  this device (system byte address)
    C_HIGHADDR            : std_logic_vector := X"60003fff";
                                          -- Maximum Address for this device (system byte 
                                          -- address xxxx3fff) (using word addressing for 
                                          -- each byte)
    C_FAMILY              : string := "virtex5";
    C_SPLB_AWIDTH         : integer := 32;
    C_SPLB_DWIDTH         : integer := 32;
    C_SPLB_P2P            : integer := 0;
    C_SPLB_MID_WIDTH      : integer := 3;
    C_SPLB_NUM_MASTERS    : integer := 8;
    C_SPLB_NATIVE_DWIDTH  : integer := 32
    );
  port (     
    PHY_tx_clk      : in std_logic;
    PHY_rx_clk      : in std_logic;
    PHY_crs         : in std_logic;
    PHY_dv          : in std_logic;
    PHY_rx_data     : in std_logic_vector (3 downto 0);
    PHY_col         : in std_logic;
    PHY_rx_er       : in std_logic;
    PHY_tx_en       : out std_logic;
    PHY_tx_data     : out std_logic_vector (3 downto 0);
        
    SPLB_Clk       : in  std_logic;
    SPLB_Rst       : in  std_logic;
    PLB_ABus       : in  std_logic_vector(0 to 31);
    PLB_UABus      : in  std_logic_vector(0 to 31);
    PLB_PAValid    : in  std_logic;
    PLB_SAValid    : in  std_logic;
    PLB_rdPrim     : in  std_logic;
    PLB_wrPrim     : in  std_logic;
    PLB_masterID   : in  std_logic_vector(0 to C_SPLB_MID_WIDTH-1);
    PLB_abort      : in  std_logic;
    PLB_busLock    : in  std_logic;
    PLB_RNW        : in  std_logic;
    PLB_BE         : in  std_logic_vector(0 to (C_SPLB_DWIDTH/8)-1);
    PLB_MSize      : in  std_logic_vector(0 to 1);
    PLB_size       : in  std_logic_vector(0 to 3);
    PLB_type       : in  std_logic_vector(0 to 2);
    PLB_lockErr    : in  std_logic;
    PLB_wrDBus     : in  std_logic_vector(0 to C_SPLB_DWIDTH-1);
    PLB_wrBurst    : in  std_logic;
    PLB_rdBurst    : in  std_logic;
    PLB_wrPendReq  : in  std_logic;
    PLB_rdPendReq  : in  std_logic;
    PLB_wrPendPri  : in  std_logic_vector(0 to 1);
    PLB_rdPendPri  : in  std_logic_vector(0 to 1);
    PLB_reqPri     : in  std_logic_vector(0 to 1);
    PLB_TAttribute : in  std_logic_vector(0 to 15);
    Sl_addrAck     : out std_logic;
    Sl_SSize       : out std_logic_vector(0 to 1);
    Sl_wait        : out std_logic;
    Sl_rearbitrate : out std_logic;
    Sl_wrDAck      : out std_logic;
    Sl_wrComp      : out std_logic;
    Sl_wrBTerm     : out std_logic;
    Sl_rdDBus      : out std_logic_vector(0 to C_SPLB_DWIDTH-1);
    Sl_rdWdAddr    : out std_logic_vector(0 to 3);
    Sl_rdDAck      : out std_logic;
    Sl_rdComp      : out std_logic;
    Sl_rdBTerm     : out std_logic;
    Sl_MBusy       : out std_logic_vector (0 to C_SPLB_NUM_MASTERS-1);
    Sl_MWrErr      : out std_logic_vector (0 to C_SPLB_NUM_MASTERS-1);
    Sl_MRdErr      : out std_logic_vector (0 to C_SPLB_NUM_MASTERS-1);
    Sl_MIRQ        : out std_logic_vector (0 to C_SPLB_NUM_MASTERS-1);
    IP2INTC_Irpt   : out std_logic
        ); 

end xemac;


architecture imp of xemac is


-------------------------------------------------------------------------------
--  Signal and Type Declarations
-------------------------------------------------------------------------------

signal ZERO_IP2IP_ADDR : std_logic_vector(0 to 12);
signal IP2bus_wrAck    : std_logic;
signal IP2bus_rdAck    : std_logic;
signal phy_rx_data_i   : std_logic_vector (0 to 3);
signal phy_tx_data_i   : std_logic_vector (0 to 3);
signal tx_DPM_ce       : std_logic;
signal tx_DPM_ce_i     : std_logic; -- added 03-03-05 MSH
signal tx_DPM_adr      : std_logic_vector (0 to 11);
signal tx_DPM_wr_data  : std_logic_vector (0 to 3);
signal tx_DPM_rd_data  : std_logic_vector (0 to 3);
signal tx_Ping_rd_data : std_logic_vector (0 to 3);
signal tx_Pong_rd_data : std_logic_vector (0 to 3) := (others => '0');
signal tx_DPM_wr_rd_n  : std_logic;              
signal rx_DPM_ce       : std_logic;
signal rx_DPM_ce_i     : std_logic; -- added 03-03-05 MSH
signal rx_DPM_adr      : std_logic_vector (0 to 11);
signal rx_DPM_wr_data  : std_logic_vector (0 to 3);
signal rx_DPM_rd_data  : std_logic_vector (0 to 3);
signal rx_Ping_rd_data : std_logic_vector (0 to 3);
signal rx_Pong_rd_data : std_logic_vector (0 to 3) := (others => '0');
signal rx_DPM_wr_rd_n  : std_logic;
signal IPIF_tx_Ping_CE : std_logic;
signal IPIF_tx_Pong_CE : std_logic := '0';
signal IPIF_rx_Ping_CE : std_logic;
signal IPIF_rx_Pong_CE : std_logic := '0';
signal tx_Ping_data_out: std_logic_vector (0 to 31);
signal tx_Pong_data_out: std_logic_vector (0 to 31) := (others => '0');
signal rx_Ping_data_out: std_logic_vector (0 to 31);
signal rx_Pong_data_out: std_logic_vector (0 to 31) := (others => '0');
signal DPM_wr_ack      : std_logic;
signal DPM_rd_ack      : std_logic;
signal rx_done         : std_logic;
signal rx_done_d1      : std_logic := '0';
signal rx_done_irq     : std_logic;
signal tx_done         : std_logic;
signal tx_done_d1      : std_logic := '0';
signal tx_done_irq     : std_logic;
signal gie             : std_logic;
signal gie_en          : std_logic;
signal gie_en_1        : std_logic;
signal gie_en_2        : std_logic;
signal gie_en_3        : std_logic;
signal tx_Ping_ce      : std_logic;
signal tx_pong_ping_l  : std_logic := '0';
signal Tx_idle         : std_logic;
signal Rx_idle         : std_logic;
signal tx_pong_tog     : std_logic := '1';
signal rx_Ping_ce      : std_logic;
signal rx_pong_ping_l  : std_logic := '0';

-------------------------------------------------------------------------------
-- New ipif_ssp1 signal declaration                                          --
-------------------------------------------------------------------------------

signal temp_Bus2IP_Addr: std_logic_vector(0 to C_SPLB_AWIDTH - 1);
signal bus2ip_addr     : std_logic_vector(0 to 12);
signal bus2ip_data     : std_logic_vector(0 to C_SPLB_NATIVE_DWIDTH  - 1);
signal bus2ip_ce       : std_logic;
signal bus2ip_rdce     : std_logic;
signal bus2ip_wrce     : std_logic;
signal bus2ip_clk      : std_logic;
signal bus2ip_reset    : std_logic;
signal ip2bus_data     : std_logic_vector(0 to C_SPLB_NATIVE_DWIDTH -1);
-------------------------------------------------------------------------------


constant C_PLB_CLK_PERIOD_PS : integer := 15000;

      --  The period of the PLB Bus clock in ps (15000 = 15ns)
-------------------------------------------------------------------------------
-- Component Declarations
-------------------------------------------------------------------------------

component SRL16E
    generic (
      INIT : bit_vector := X"0000"
      );

  port  (
    Q   : out std_logic;--[out]
    A0  : in  std_logic; --[in]
    A1  : in  std_logic; --[in]
    A2  : in  std_logic; --[in]
    A3  : in  std_logic; --[in]
    CE  : in  std_logic; --[in]
    CLK : in  std_logic; --[in]
    D   : in  std_logic  --[in]
  );
end component;


component FDR
  port (
    Q : out std_logic;
    C : in std_logic;
    D : in std_logic;
    R : in std_logic
  );
end component;

component FDRE
  port (
    Q  : out std_logic;
    C  : in std_logic;
    CE : in std_logic;
    D  : in std_logic;
    R  : in std_logic
  );
end component;

   component LUT4
      generic(INIT : bit_vector);
      port (
        O  : out std_logic;
        I0 : in std_logic;
        I1 : in std_logic;
        I2 : in std_logic;
        I3 : in std_logic
      );
   end component;

   
begin
  
  ZERO_IP2IP_ADDR <= (others => '0');
  
  IP2INTC_Irpt <= (rx_done_irq and gie) or (tx_done_irq and gie);
-------------------------------------------------------------------------------
-- IPIF interface 
-------------------------------------------------------------------------------
XPS_IPIF_I : entity xps_ethernetlite_v2_00_c.xps_ipif_ssp1 
  generic map 
  (
        C_FAMILY             => C_FAMILY,
        C_SPLB_AWIDTH        => C_SPLB_AWIDTH,
        C_SPLB_DWIDTH        => C_SPLB_DWIDTH,
        C_SPLB_P2P           => C_SPLB_P2P,
        C_SPLB_MID_WIDTH     => C_SPLB_MID_WIDTH,
        C_SPLB_NUM_MASTERS   => C_SPLB_NUM_MASTERS,
        C_SPLB_NATIVE_DWIDTH => C_SPLB_NATIVE_DWIDTH ,
        C_BASEADDR           => C_BASEADDR,
        C_HIGHADDR           => C_HIGHADDR
  )
  port  map
  (
          SPLB_Clk      => SPLB_Clk,
          SPLB_Rst      => SPLB_Rst,
          PLB_ABus      => PLB_ABus,
          PLB_UABus     => PLB_UABus,
          PLB_PAValid   => PLB_PAValid,
          PLB_SAValid   => PLB_SAValid,
          PLB_rdPrim    => PLB_rdPrim,
          PLB_wrPrim    => PLB_wrPrim,
          PLB_masterID  => PLB_masterID,
          PLB_abort     => PLB_abort,
          PLB_busLock   => PLB_busLock,
          PLB_RNW       => PLB_RNW,
          PLB_BE        => PLB_BE,
          PLB_MSize     => PLB_MSize,
          PLB_size      => PLB_size,
          PLB_type      => PLB_type,
          PLB_lockErr   => PLB_lockErr,
          PLB_wrDBus    => PLB_wrDBus,
          PLB_wrBurst   => PLB_wrBurst,
          PLB_rdBurst   => PLB_rdBurst,
          PLB_wrPendReq => PLB_wrPendReq,
          PLB_rdPendReq => PLB_rdPendReq,
          PLB_wrPendPri => PLB_wrPendPri,
          PLB_rdPendPri => PLB_rdPendPri,
          PLB_reqPri    => PLB_reqPri,
          PLB_TAttribute=> PLB_TAttribute,
          Sl_addrAck    => Sl_addrAck,
          Sl_SSize      => Sl_SSize,
          Sl_wait       => Sl_wait,
          Sl_rearbitrate=> Sl_rearbitrate,
          Sl_wrDAck     => Sl_wrDAck,
          Sl_wrComp     => Sl_wrComp,
          Sl_wrBTerm    => Sl_wrBTerm,
          Sl_rdDBus     => Sl_rdDBus,
          Sl_rdWdAddr   => Sl_rdWdAddr,
          Sl_rdDAck     => Sl_rdDAck,
          Sl_rdComp     => Sl_rdComp,
          Sl_rdBTerm    => Sl_rdBTerm,
          Sl_MBusy      => Sl_MBusy,
          Sl_MWrErr     => Sl_MWrErr,
          Sl_MRdErr     => Sl_MRdErr,
          Sl_MIRQ       => Sl_MIRQ,
          Bus2IP_Clk    => Bus2IP_Clk,
          Bus2IP_Reset  => Bus2IP_Reset,
          IP2Bus_Data   => IP2Bus_Data,
          Bus2IP_Data   => Bus2IP_Data,
          Bus2IP_Addr   => temp_Bus2IP_Addr,
          Bus2IP_RdCE   => Bus2IP_RdCE,
          Bus2IP_WrCE   => Bus2IP_WrCE,
          IP2Bus_WrAck  => IP2Bus_WrAck,
          IP2Bus_RdAck  => IP2Bus_RdAck
  );
  
bus2ip_add_gen:for i in 0 to 12 generate
          bus2ip_addr(i) <=temp_Bus2IP_Addr((temp_Bus2IP_Addr'high - 12) + i);
               end generate;

  phy_tx_data(0) <= phy_tx_data_i(3);
  phy_tx_data(1) <= phy_tx_data_i(2);
  phy_tx_data(2) <= phy_tx_data_i(1);
  phy_tx_data(3) <= phy_tx_data_i(0);                 
  
  phy_rx_data_i(0) <= phy_rx_data(3);
  phy_rx_data_i(1) <= phy_rx_data(2);
  phy_rx_data_i(2) <= phy_rx_data(1);
  phy_rx_data_i(3) <= phy_rx_data(0);      

-------------------------------------------------------------------------------
-- EMAC
-------------------------------------------------------------------------------   
  EMAC_I: entity xps_ethernetlite_v2_00_c.emac
    generic map (
      C_DUPLEX       => C_DUPLEX,
      NODE_MAC       => NODE_MAC,
      C_FAMILY       => C_FAMILY  
      )
    port map (    
      phy_tx_clk     => phy_tx_clk,
      phy_rx_clk     => phy_rx_clk,
      phy_crs        => phy_crs,
      phy_dv         => phy_dv,
      phy_rx_data    => phy_rx_data_i,
      phy_col        => phy_col,
      phy_rx_er      => phy_rx_er,
      
      phy_tx_en      => phy_tx_en,
      phy_tx_data    => phy_tx_data_i,
      
      tx_DPM_ce      => tx_DPM_ce_i, -- changed 03-03-05
      tx_DPM_adr     => tx_DPM_adr,
      tx_DPM_wr_data => tx_DPM_wr_data,
      tx_DPM_rd_data => tx_DPM_rd_data,
      tx_DPM_wr_rd_n => tx_DPM_wr_rd_n,
      tx_done        => tx_done,
      tx_done_irq    => tx_done_irq,
      tx_pong_ping_l => tx_pong_ping_l,
      Tx_idle        => Tx_idle,
      Rx_idle        => Rx_idle,
      rx_DPM_ce      => rx_DPM_ce_i, -- changed 03-03-05
      rx_DPM_adr     => rx_DPM_adr,
      rx_DPM_wr_data => rx_DPM_wr_data,
      rx_DPM_rd_data => rx_DPM_rd_data,
      rx_DPM_wr_rd_n => rx_DPM_wr_rd_n ,
      rx_done        => rx_done,
      rx_done_irq    => rx_done_irq,
      rx_pong_ping_l => rx_pong_ping_l,
                             
      rst            => Bus2IP_Reset,
      clk            => Bus2IP_Clk
      );

  bus2ip_ce <= Bus2IP_RdCE or Bus2IP_WrCE;
  IPIF_tx_Ping_CE <= bus2ip_ce   and not Bus2IP_Addr(0) 
                                    and not Bus2IP_Addr(1);
  IPIF_rx_Ping_CE <= bus2ip_ce   and     Bus2IP_Addr(0) 
                                    and not Bus2IP_Addr(1);
  IP2Bus_WrAck    <= DPM_wr_ack;
  IP2Bus_RdAck    <= DPM_rd_ack;


   IP2BUS_DATA_GENERATE: for i in 0 to 31 generate
      IP2Bus_Data(i) <= (tx_Ping_data_out(i) and IPIF_tx_Ping_CE) or
                        (tx_Pong_data_out(i) and IPIF_tx_Pong_CE) or
                        (rx_Ping_data_out(i) and IPIF_rx_Ping_CE) or
                        (rx_Pong_data_out(i) and IPIF_rx_Pong_CE);
   end generate IP2BUS_DATA_GENERATE;   
  
  RD_ACK_PROCESS:process (Bus2IP_Clk)
  begin
    if (Bus2IP_Clk'event and Bus2IP_Clk = '1') then
      if (Bus2IP_Reset = '1') then
        DPM_rd_ack <= '0';
      elsif (Bus2IP_RdCE = '1' and DPM_rd_ack = '0') then
        DPM_rd_ack <= '1';
      else
        DPM_rd_ack <= '0';
      end if;
    end if;
  end process;   

  WR_ACK_PROCESS:process (Bus2IP_Clk)
  begin
    if (Bus2IP_Clk'event and Bus2IP_Clk = '1') then
      if (Bus2IP_Reset = '1') then
        DPM_wr_ack <= '0';
      elsif (Bus2IP_WrCE = '1' and DPM_wr_ack = '0') then
        DPM_wr_ack <= '1';
      else
        DPM_wr_ack <= '0';
      end if;
    end if;
  end process;   

   tx_ping_ce <= tx_DPM_ce and not tx_pong_ping_l;
   
 
   DPM_TX_RD_DATA_GENERATE: for i in 0 to 3 generate
      tx_DPM_rd_data(i) <= (tx_ping_rd_data(i) and not tx_pong_ping_l 
                                           and (tx_pong_tog or not Tx_idle)) or
                           (tx_pong_rd_data(i) and     tx_pong_ping_l
                                           and (tx_pong_tog or not Tx_idle));
   end generate DPM_TX_RD_DATA_GENERATE; 

   DPM_RX_RD_DATA_GENERATE: for i in 0 to 3 generate
      rx_DPM_rd_data(i) <= (rx_ping_rd_data(i) and not rx_pong_ping_l) or
                           (rx_pong_rd_data(i) and     rx_pong_ping_l);
   end generate DPM_RX_RD_DATA_GENERATE;
   
   tx_DPM_ce <= tx_DPM_ce_i;
   rx_DPM_ce <= rx_DPM_ce_i;
   

  TX_PING: entity xps_ethernetlite_v2_00_c.emac_dpram
    generic map (
      C_FAMILY             => C_FAMILY
      )
    port map (   
      ce_a                 => tx_ping_ce                 , --[in]
      wr_rd_n_a            => tx_DPM_wr_rd_n             , --[in]
      adr_a                => tx_DPM_adr                 , --[in]
      data_in_a            => tx_DPM_wr_data             , --[in]
      data_out_a           => tx_ping_rd_data            , --[out]
      ce_b                 => IPIF_tx_Ping_CE            , --[in]  
      wr_rd_n_b            => Bus2IP_WrCE                , --[in]
      adr_b                => Bus2IP_Addr(2 to 10)       , --[in]
      data_in_b( 0 to  7)  => Bus2IP_Data(24 to 31)      , --[in]
      data_in_b( 8 to 15)  => Bus2IP_Data(16 to 23)      , --[in]
      data_in_b(16 to 23)  => Bus2IP_Data( 8 to 15)      , --[in]
      data_in_b(24 to 31)  => Bus2IP_Data( 0 to  7)      , --[in]
      data_out_b( 0 to  7) => tx_ping_data_out(24 to 31) , --[out]
      data_out_b( 8 to 15) => tx_ping_data_out(16 to 23) , --[out]
      data_out_b(16 to 23) => tx_ping_data_out( 8 to 15) , --[out]
      data_out_b(24 to 31) => tx_ping_data_out( 0 to  7) , --[out]
      clk                  => Bus2IP_Clk                 ,
      rst                  => Bus2IP_Reset
      );          
  
   rx_ping_ce <= rx_DPM_ce and not rx_pong_ping_l;
   
  RX_PING: entity xps_ethernetlite_v2_00_c.emac_dpram
    generic map (
      C_FAMILY             => C_FAMILY
      )
    port map (         
      ce_a                 => rx_ping_ce                 ,  
      wr_rd_n_a            => rx_DPM_wr_rd_n             , 
      adr_a                => rx_DPM_adr                 ,  
      data_in_a            => rx_DPM_wr_data             , 
      data_out_a           => rx_ping_rd_data            , 
      ce_b                 => IPIF_rx_Ping_CE            ,
      wr_rd_n_b            => Bus2IP_WrCE                ,
      adr_b                => Bus2IP_Addr(2 to 10)       ,
      data_in_b( 0 to  7)  => Bus2IP_Data(24 to 31)      ,
      data_in_b( 8 to 15)  => Bus2IP_Data(16 to 23)      ,
      data_in_b(16 to 23)  => Bus2IP_Data( 8 to 15)      ,
      data_in_b(24 to 31)  => Bus2IP_Data( 0 to  7)      ,
      data_out_b( 0 to  7) => rx_Ping_data_out(24 to 31) ,
      data_out_b( 8 to 15) => rx_Ping_data_out(16 to 23) ,
      data_out_b(16 to 23) => rx_Ping_data_out( 8 to 15) ,
      data_out_b(24 to 31) => rx_Ping_data_out( 0 to  7) ,
      clk                  => Bus2IP_Clk                 ,
      rst                  => Bus2IP_Reset 
      );
      
-------------------------------------------------------------------------------
-- Transmit Ping control signal generation 
-------------------------------------------------------------------------------

TX_PING_SIG: if C_TX_PING_PONG = 0 generate

             tx_pong_ping_l <= '0' ;
             tx_pong_tog    <= '1' ;
             
    end generate TX_PING_SIG;
    
-------------------------------------------------------------------------------
-- Transmit Pong memory generate
-------------------------------------------------------------------------------
TX_PONG_GEN: if C_TX_PING_PONG = 1 generate

   signal tx_pong_ce        : std_logic;
   signal pp_tog_ce         : std_logic;

   attribute INIT                  : string;

   attribute INIT of PP_TOG_LUT_I: label is "1111";

   Begin
   
     TX_PONG_I: entity xps_ethernetlite_v2_00_c.emac_dpram
       generic map (
         C_FAMILY             => C_FAMILY
         )                    
       port map (             
         ce_a                 => tx_pong_ce                 , --tx_pong_ping_l             ,
         wr_rd_n_a            => tx_DPM_wr_rd_n             ,
         adr_a                => tx_DPM_adr                 ,
         data_in_a            => tx_DPM_wr_data             ,
         data_out_a           => tx_Pong_rd_data            ,
         ce_b                 => IPIF_tx_Pong_CE            ,      
         wr_rd_n_b            => Bus2IP_WrCE                ,
         adr_b                => Bus2IP_Addr(2 to 10)       ,
         data_in_b( 0 to  7)  => Bus2IP_Data(24 to 31)      ,
         data_in_b( 8 to 15)  => Bus2IP_Data(16 to 23)      ,
         data_in_b(16 to 23)  => Bus2IP_Data( 8 to 15)      ,
         data_in_b(24 to 31)  => Bus2IP_Data( 0 to  7)      ,
         data_out_b( 0 to  7) => tx_Pong_data_out(24 to 31) ,
         data_out_b( 8 to 15) => tx_Pong_data_out(16 to 23) ,
         data_out_b(16 to 23) => tx_Pong_data_out( 8 to 15) ,
         data_out_b(24 to 31) => tx_Pong_data_out( 0 to  7) ,
         clk                  => Bus2IP_Clk                 ,
         rst                  => Bus2IP_Reset
         );          
   
   tx_pong_ce <= tx_DPM_ce and tx_pong_ping_l;


TX_DONE_D1_I: FDR
  port map (
    Q => tx_done_d1 , --[out]
    C => Bus2IP_Clk , --[in]
    D => tx_done    , --[in]
    R => Bus2IP_Reset --[in]
  );


     TX_PONG_PING_L_PROCESS:process (Bus2IP_Clk)
      begin   -- process
         if (Bus2IP_Clk'event and Bus2IP_Clk = '1') then
            if (Bus2IP_Reset = '1') then
               tx_pong_ping_l <= '0';
            elsif (tx_done_d1 = '1' and tx_pong_ping_l = '0') or
                  (tx_pong_tog = '1' and tx_DPM_rd_data(3)  = '0' and
                  tx_pong_ping_l = '0') then
                  tx_pong_ping_l <= '1';
            elsif (tx_done_d1 = '1' and tx_pong_ping_l = '1') or
                  (tx_pong_tog = '1' and tx_DPM_rd_data(3)  = '0' and
                  tx_pong_ping_l = '1') then
                  tx_pong_ping_l <= '0';
            else
               tx_pong_ping_l <= tx_pong_ping_l;
            end if;
         end if;
      end process;   

  IPIF_tx_Pong_CE <= bus2ip_ce and not Bus2IP_Addr(0) 
                                    and Bus2IP_Addr(1);

 PP_TOG_LUT_I: SRL16E
-- synthesis translate_off 
    generic map ( 
      INIT => X"1111") 
-- synthesis translate_on 
  port map (
    Q   => tx_pong_tog, --[out]
    A0  => '1'        , --[in]
    A1  => '1'        , --[in]
    A2  => '1'        , --[in]
    A3  => '1'        , --[in]
    CE  => pp_tog_ce  , --[in]
    CLK => Bus2IP_Clk , --[in]
    D   => tx_pong_tog  --[in]
  );

  pp_tog_ce <= (Tx_idle and not tx_DPM_rd_data(3)) or tx_pong_tog;

   end generate TX_PONG_GEN; 
   
-------------------------------------------------------------------------------
-- Receivet Ping control signal generation 
-------------------------------------------------------------------------------

RX_PING_SIG: if C_RX_PING_PONG = 0 generate

             rx_pong_ping_l <= '0' ;
                        
    end generate RX_PING_SIG;   

-------------------------------------------------------------------------------
-- Receive Pong memory generate
-------------------------------------------------------------------------------
RX_PONG_GEN: if C_RX_PING_PONG = 1 generate

   signal rx_pong_ce        : std_logic;

   Begin
   
     RX_PONG_I: entity xps_ethernetlite_v2_00_c.emac_dpram
       generic map (
         C_FAMILY             => C_FAMILY
         )                    
       port map (             
         ce_a                 => rx_pong_ce                 , --rx_pong_ping_l             ,
         wr_rd_n_a            => rx_DPM_wr_rd_n             ,
         adr_a                => rx_DPM_adr                 ,
         data_in_a            => rx_DPM_wr_data             ,
         data_out_a           => rx_Pong_rd_data            ,
         ce_b                 => IPIF_rx_Pong_CE            ,      
         wr_rd_n_b            => Bus2IP_WrCE                ,
         adr_b                => Bus2IP_Addr(2 to 10)       ,
         data_in_b( 0 to  7)  => Bus2IP_Data(24 to 31)      ,
         data_in_b( 8 to 15)  => Bus2IP_Data(16 to 23)      ,
         data_in_b(16 to 23)  => Bus2IP_Data( 8 to 15)      ,
         data_in_b(24 to 31)  => Bus2IP_Data( 0 to  7)      ,
         data_out_b( 0 to  7) => rx_Pong_data_out(24 to 31) ,
         data_out_b( 8 to 15) => rx_Pong_data_out(16 to 23) ,
         data_out_b(16 to 23) => rx_Pong_data_out( 8 to 15) ,
         data_out_b(24 to 31) => rx_Pong_data_out( 0 to  7) ,
         clk                  => Bus2IP_Clk                 ,
         rst                  => Bus2IP_Reset
         );          
   
   rx_pong_ce <= rx_DPM_ce and rx_pong_ping_l;


RX_DONE_D1_I: FDR
  port map (
    Q => rx_done_d1 , --[out]
    C => Bus2IP_Clk , --[in]
    D => rx_done    , --[in]
    R => Bus2IP_Reset --[in]
  );


     RX_PONG_PING_L_PROCESS:process (Bus2IP_Clk)
      begin   -- process
         if (Bus2IP_Clk'event and Bus2IP_Clk = '1') then
            if (Bus2IP_Reset = '1') then
               rx_pong_ping_l <= '0';
            elsif (rx_done_d1 = '1') then
               if rx_pong_ping_l = '0' then
                  rx_pong_ping_l <= '1';
               else
                  rx_pong_ping_l <= '0';
              end if;
            else
               rx_pong_ping_l <= rx_pong_ping_l;
            end if;
         end if;
      end process;   

  IPIF_rx_Pong_CE <= bus2ip_ce   and Bus2IP_Addr(0) 
                                    and Bus2IP_Addr(1);

   end generate RX_PONG_GEN; 

-------------------------------------------------------------------------------
-- Global Interrupt Enable Register
-------------------------------------------------------------------------------
GIE_I: FDRE
  port map (
    Q => gie               , --[out]
    C => SPLB_Clk          , --[in]
    CE => gie_en           , --[in]
    D => Bus2IP_Data(0)   , --[in]
    R => Bus2IP_Reset        --[in]
  );



    GIE_EN_1_I : LUT4
      generic map(
        INIT => X"8000"
        )
      port map (
        O  => gie_en_1         ,   -- [out]
        I0 => Bus2IP_Addr(5)   ,   -- [in]
        I1 => Bus2IP_Addr(4)   ,   -- [in]
        I2 => Bus2IP_Addr(3)   ,   -- [in]
        I3 => Bus2IP_Addr(2)       -- [in]
                );  
    GIE_EN_2_I : LUT4
      generic map(
        INIT => X"8000"
        )
      port map (
        O  => gie_en_2         ,   -- [out]
        I0 => Bus2IP_Addr(9)   ,   -- [in]
        I1 => Bus2IP_Addr(8)   ,   -- [in]
        I2 => Bus2IP_Addr(7)   ,   -- [in]
        I3 => Bus2IP_Addr(6)       -- [in]
                );  
    GIE_EN_3_I : LUT4
      generic map(
        INIT => X"0100"
        )
      port map (
        O  => gie_en_3         ,   -- [out]
        I0 => '0'              ,   -- [in]
        I1 => '0'              ,   -- [in]
        I2 => Bus2IP_Addr(10)  ,   -- [in]
        I3 => Bus2IP_WrCE          -- [in]
                );  
    GIE_EN_I : LUT4
      generic map(
        INIT => X"8000"
        )
      port map (
        O  => gie_en           ,   -- [out]
        I0 => IPIF_tx_Ping_CE  ,   -- [in]
        I1 => gie_en_3         ,   -- [in]
        I2 => gie_en_2         ,   -- [in]
        I3 => gie_en_1             -- [in]
                );  

end imp;
