-------------------------------------------------------------------------------
-- $Id: xps_ipif_ssp1.vhd,v 1.1 2007/12/11 06:41:19 sanjayk Exp $
-------------------------------------------------------------------------------
--  xps_ipif_ssp1 - entity/architecture pair
-------------------------------------------------------------------------------
--  ***************************************************************************
--  **  Copyright(C) 2006 by Xilinx, Inc. All rights reserved.               **
--  **                                                                       **
--  **  This text contains proprietary, confidential                         **
--  **  information of Xilinx, Inc. , is distributed by                      **
--  **  under license from Xilinx, Inc., and may be used,                    **
--  **  copied and/or disclosed only pursuant to the terms                   **
--  **  of a valid license agreement with Xilinx, Inc.                       **
--  **                                                                       **
--  **  Unmodified source code is guaranteed to place and route,             **
--  **  function and run at speed according to the datasheet                 **
--  **  specification. Source code is provided "as-is", with no              **
--  **  obligation on the part of Xilinx to provide support.                 **
--  **                                                                       **
--  **  Xilinx Hotline support of source code IP shall only include          **
--  **  standard level Xilinx Hotline support, and will only address         **
--  **  issues and questions related to the standard released Netlist        **
--  **  version of the core (and thus indirectly, the original core source). **
--  **                                                                       **
--  **  The Xilinx Support Hotline does not have access to source            **
--  **  code and therefore cannot answer specific questions related          **
--  **  to source HDL. The Xilinx Support Hotline will only be able          **
--  **  to confirm the problem in the Netlist version of the core.           **
--  **                                                                       **
--  **  This copyright and support notice must be retained as part           **
--  **  of this text at all times.                                           **
--  ***************************************************************************
--
-------------------------------------------------------------------------------
-- Filename:        xps_ipif_ssp1.vhd
-- 
-- Description:     
--                  This file contains the IPIF V3_01_a wrapper which passes 
--                  generics to IPIF inorder to customize it for Ethernetlite
--                  core
-- VHDL-Standard:   VHDL'93
-------------------------------------------------------------------------------
-- Structure:   
--
--  xps_ethernetlite.vhd
--      \
--      \-- xemac.vhd
--           \
--           \--xps_ipif_ssp1.vhd
--           \
--           \-- emac_dpram.vhd                     
--           \    \                     
--           \    \-- RAMB16_S36_S36
--           \    \     or
--           \    \-- RAMB4_S8_S8
--           \                          
--           \-- emac.vhd                     
--                \                     
--                \                     
--                \-- receive.vhd
--                \      rx_state_2.vhd
--                \      rx_intrfce.vhd
--                \         veasyncfifo5w15ddstrb.edn
--                \            or
--                \         v2asyncfifo5w15ddstrb.edn
--                \      cntr4bit.vhd
--                \      nibshiftin.vhd
--                \      nibshiftdain.vhd
--                \      cntr11bit.vhd
--                \      cntr12bit.vhd
--                \      dacheck.vhd
--                \      rxlengthcntr.vhd
--                \      crcgenrx.vhd
--                \      rxcrcnibcnt.vhd
--                \                     
--                \-- transmit_2.vhd
--                       crcgentx.vhd
--                          crcnibshiftreg
--                       tx_intrfce.vhd
--                          veasyncfifo5w15ddstrb.edn
--                             or
--                          v2asyncfifo5w15ddstrb.edn
--                       cntr11bit.vhd
--                       nibshiftout.vhd
--                       tx_state_3.vhd
--                       deferral.vhd
--                          cntr5bit.vhd
--                          defer_state.vhd
--                       bocntr.vhd
--                          lfsr16.vhd
--                       msh_cnt.vhd
--                       ld_arith_reg.vhd
-------------------------------------------------------------------------------
-- Author:         Vaibhav
-- History:
--  RSK            11/06/06
-- ^^^^^^
--                 First Version, Based on opb_ethernetlite v1.01.b
--  ~~~~~
-------------------------------------------------------------------------------
-- Naming Conventions:
--      active low signals:                     "*_n"
--      clock signals:                          "clk", "clk_div#", "clk_#x" 
--      reset signals:                          "rst", "rst_n" 
--      generics:                               "C_*" 
--      user defined types:                     "*_TYPE" 
--      state machine next state:               "*_ns" 
--      state machine current state:            "*_cs" 
--      combinatorial signals:                  "*_com" 
--      pipelined or register delay signals:    "*_d#" 
--      counter signals:                        "*cnt*"
--      clock enable signals:                   "*_ce" 
--      internal version of output port         "*_i"
--      device pins:                            "*_pin" 
--      ports:                                  - Names begin with Uppercase 
--      processes:                              "*_PROCESS" 
--      component instantiations:               "<ENTITY_>I_<#|FUNC>
-------------------------------------------------------------------------------
library ieee;
library plbv46_slave_single_v1_00_a;

library proc_common_v2_00_a;
use proc_common_v2_00_a.proc_common_pkg.all;
use proc_common_v2_00_a.ipif_pkg.all;

use ieee.std_logic_1164.all;
use plbv46_slave_single_v1_00_a.all;

-------------------------------------------------------------------------------  
-- ENTITY
-------------------------------------------------------------------------------

entity xps_ipif_ssp1 is
  generic
  (
        C_FAMILY              : string  := "virtex5";
        C_SPLB_AWIDTH         : integer := 32;
        C_SPLB_DWIDTH         : integer := 32;
        C_SPLB_P2P            : integer := 0;
        C_SPLB_MID_WIDTH      : integer := 3;
        C_SPLB_NUM_MASTERS    : integer := 8;
        C_SPLB_NATIVE_DWIDTH  : integer := 32;
        C_BASEADDR            : std_logic_vector(0 to 31);
        C_HIGHADDR            : std_logic_vector(0 to 31)

  );
  port
  (
        SPLB_Clk       : in  std_logic;
        SPLB_Rst       : in  std_logic;
        PLB_ABus       : in  std_logic_vector(0 to 31);
        PLB_UABus      : in  std_logic_vector(0 to 31);
        PLB_PAValid    : in  std_logic;
        PLB_SAValid    : in  std_logic;
        PLB_rdPrim     : in  std_logic;
        PLB_wrPrim     : in  std_logic;
        PLB_masterID   : in  std_logic_vector(0 to C_SPLB_MID_WIDTH-1);
        PLB_abort      : in  std_logic;
        PLB_busLock    : in  std_logic;
        PLB_RNW        : in  std_logic;
        PLB_BE         : in  std_logic_vector(0 to (C_SPLB_DWIDTH/8)-1);
        PLB_MSize      : in  std_logic_vector(0 to 1);
        PLB_size       : in  std_logic_vector(0 to 3);
        PLB_type       : in  std_logic_vector(0 to 2);
        PLB_lockErr    : in  std_logic;
        PLB_wrDBus     : in  std_logic_vector(0 to C_SPLB_DWIDTH-1);
        PLB_wrBurst    : in  std_logic;
        PLB_rdBurst    : in  std_logic;
        PLB_wrPendReq  : in  std_logic;
        PLB_rdPendReq  : in  std_logic;
        PLB_wrPendPri  : in  std_logic_vector(0 to 1);
        PLB_rdPendPri  : in  std_logic_vector(0 to 1);
        PLB_reqPri     : in  std_logic_vector(0 to 1);
        PLB_TAttribute : in  std_logic_vector(0 to 15);
        Sl_addrAck     : out std_logic;
        Sl_SSize       : out std_logic_vector(0 to 1);
        Sl_wait        : out std_logic;
        Sl_rearbitrate : out std_logic;
        Sl_wrDAck      : out std_logic;
        Sl_wrComp      : out std_logic;
        Sl_wrBTerm     : out std_logic;
        Sl_rdDBus      : out std_logic_vector(0 to C_SPLB_DWIDTH-1);
        Sl_rdWdAddr    : out std_logic_vector(0 to 3);
        Sl_rdDAck      : out std_logic;
        Sl_rdComp      : out std_logic;
        Sl_rdBTerm     : out std_logic;
        Sl_MBusy       : out std_logic_vector (0 to C_SPLB_NUM_MASTERS-1);
        Sl_MWrErr      : out std_logic_vector (0 to C_SPLB_NUM_MASTERS-1);
        Sl_MRdErr      : out std_logic_vector (0 to C_SPLB_NUM_MASTERS-1);
        Sl_MIRQ        : out std_logic_vector (0 to C_SPLB_NUM_MASTERS-1);
        Bus2IP_Clk     : out std_logic;        
        Bus2IP_Reset   : out std_logic;
        IP2Bus_Data    : in  std_logic_vector(0 to C_SPLB_NATIVE_DWIDTH -1);
        Bus2IP_Data    : out std_logic_vector(0 to C_SPLB_NATIVE_DWIDTH -1);
        Bus2IP_Addr    : out std_logic_vector(0 to C_SPLB_AWIDTH-1);
        Bus2IP_RdCE    : out std_logic;
        Bus2IP_WrCE    : out std_logic;
        IP2Bus_WrAck   : in  std_logic;
        IP2Bus_RdAck   : in  std_logic
        );
end entity xps_ipif_ssp1;

-------------------------------------------------------------------------------
-- architecture
-------------------------------------------------------------------------------

architecture imp of xps_ipif_ssp1 is

-------------------  Constant Declaration Section BEGIN -----------------------

constant ZERO_ADDR_PAD            : std_logic_vector(0 to 64-C_SPLB_AWIDTH-1)
                                    := (others => '0');
constant PIPELINE_MODEL           : integer   := 4;  
constant USER_NUM_CE              : integer     := 1;
constant ARD_ID_ARRAY             : INTEGER_ARRAY_TYPE := (0 => USER_00);
constant POSTED_ZEROS             : std_logic_vector(0 to ARD_ID_ARRAY'length-1)
                                    := (others => '0');
constant ARD_ADDR_RANGE_ARRAY     : SLV64_ARRAY_TYPE :=
                                    (ZERO_ADDR_PAD & C_BASEADDR, 
                                     ZERO_ADDR_PAD & C_HIGHADDR);
constant ARD_NUM_CE_ARRAY         : INTEGER_ARRAY_TYPE := (0 => 1);

-- No dependent properties
constant ARD_DEPENDENT_PROPS_ARRAY: DEPENDENT_PROPS_ARRAY_TYPE 
                                    :=  (0 => (others => 0));

constant IP_INTR_MODE_ARRAY       : INTEGER_ARRAY_TYPE :=  (0=>1);
      
-- Do not include MIR
constant DEV_MIR_ENABLE           : integer := 0;
constant DEV_BLK_ID               : integer := 1;
-- Burst support generics
constant INCLUDE_BURST_SUPPORT    : integer   := 0; 
-- need both the address counter and the write burst buffer
constant INCLUDE_ADDR_CNTR        : integer := 0;
constant INCLUDE_WR_BUF           : integer := 0;

constant ZERO_DATA                : std_logic_vector
                                    (0 to C_SPLB_NATIVE_DWIDTH -1)
                                    :=(others => '0');
constant ZERO_INTR                : std_logic_vector
                                    (0 to IP_INTR_MODE_ARRAY'length-1)
                                    := (others => '0');
    
-------------------  Constant Declaration Section END -------------------------
signal ssp1_Bus2IP_CE  :std_logic_vector(0 to calc_num_ce(ARD_NUM_CE_ARRAY)-1);
signal ssp1_Bus2IP_RdCE:std_logic_vector(0 to calc_num_ce(ARD_NUM_CE_ARRAY)-1);
signal ssp1_Bus2IP_WrCE:std_logic_vector(0 to calc_num_ce(ARD_NUM_CE_ARRAY)-1);
-------------------------------------------------------------------------------

begin

  Bus2IP_RdCE <= ssp1_Bus2IP_RdCE(0);
  Bus2IP_WrCE <= ssp1_Bus2IP_WrCE(0);

PLBV46_I : entity plbv46_slave_single_v1_00_a.plbv46_slave_single
         
        generic map (
        
       C_ARD_ADDR_RANGE_ARRAY  => ARD_ADDR_RANGE_ARRAY,
       C_ARD_NUM_CE_ARRAY      => ARD_NUM_CE_ARRAY,
       C_SPLB_P2P              => C_SPLB_P2P,
       C_SPLB_MID_WIDTH        => C_SPLB_MID_WIDTH,
       C_SPLB_NUM_MASTERS      => C_SPLB_NUM_MASTERS,
       C_SPLB_AWIDTH           => C_SPLB_AWIDTH,
       C_SPLB_DWIDTH           => C_SPLB_DWIDTH,
       C_SIPIF_DWIDTH          => C_SPLB_NATIVE_DWIDTH ,
       C_FAMILY                => C_FAMILY
       )
       
    port map(
    -- System signals
        SPLB_Clk               => SPLB_Clk,
        SPLB_Rst               => SPLB_Rst,
    -- Bus Slave signals
        PLB_ABus               => PLB_ABus,
        PLB_UABus              => PLB_UABus,
        PLB_PAValid            => PLB_PAValid,
        PLB_SAValid            => PLB_SAValid,
        PLB_rdPrim             => PLB_rdPrim,
        PLB_wrPrim             => PLB_wrPrim,
        PLB_masterID           => PLB_masterID,
        PLB_abort              => PLB_abort,
        PLB_busLock            => PLB_busLock,
        PLB_RNW                => PLB_RNW,
        PLB_BE                 => PLB_BE,
        PLB_MSize              => PLB_MSize,
        PLB_size               => PLB_size,
        PLB_type               => PLB_type,
        PLB_lockErr            => PLB_lockErr,
        PLB_wrDBus             => PLB_wrDBus,
        PLB_wrBurst            => PLB_wrBurst,
        PLB_rdBurst            => PLB_rdBurst,
        PLB_wrPendReq          => PLB_wrPendReq,
        PLB_rdPendReq          => PLB_rdPendReq,
        PLB_wrPendPri          => PLB_wrPendPri,
        PLB_rdPendPri          => PLB_rdPendPri,
        PLB_reqPri             => PLB_reqPri,
        PLB_TAttribute         => PLB_TAttribute,
    -- Slave Responce Signals
        Sl_addrAck             => Sl_addrAck,
        Sl_SSize               => Sl_SSize,
        Sl_wait                => Sl_wait,
        Sl_rearbitrate         => Sl_rearbitrate,
        Sl_wrDAck              => Sl_wrDAck,
        Sl_wrComp              => Sl_wrComp,
        Sl_wrBTerm             => Sl_wrBTerm,
        Sl_rdDBus              => Sl_rdDBus,
        Sl_rdWdAddr            => Sl_rdWdAddr,
        Sl_rdDAck              => Sl_rdDAck,
        Sl_rdComp              => Sl_rdComp,
        Sl_rdBTerm             => Sl_rdBTerm,
        Sl_MBusy               => Sl_MBusy,
        Sl_MWrErr              => Sl_MWrErr,
        Sl_MRdErr              => Sl_MRdErr,
        Sl_MIRQ                => Sl_MIRQ,
    -- IP Interconnect (IPIC) port signals
        Bus2IP_Clk             => Bus2IP_Clk,
        Bus2IP_Reset           => Bus2IP_Reset,
        IP2Bus_Data            => IP2Bus_Data,
        IP2Bus_WrAck           => IP2Bus_WrAck,
        IP2Bus_RdAck           => IP2Bus_RdAck,
        IP2Bus_Error           => '0',
        Bus2IP_Addr            => Bus2IP_Addr,
        Bus2IP_Data            => Bus2IP_Data,
        Bus2IP_RNW             => open,
        Bus2IP_BE              => open,
        Bus2IP_CS              => open,
        Bus2IP_RdCE            => ssp1_Bus2IP_RdCE,
        Bus2IP_WrCE            => ssp1_Bus2IP_WrCE
       );

end architecture imp;