#ifndef WARPNET_H
#define WARPNET_H

//WARPnet Stuff

//Number of PPC clk cycles between stats updates
#define WARPNET_STATSPKT_PERIOD 60000000

//Index of OFDM PHY pkt buffer used for sending WARPnet Ethernet packets
#define WARPNET_PKTBUFFINDEX 4

#define WARPNET_NUMNODES 3

//The structs/defines below must match the definitions at the WARPNet server!

//Struct representing raw Ethernet header + packet contents
typedef struct {
	//Standard 14-byte Ethernet header (dest/src MAC addresses + EtherType)
	unsigned char dstAddr[6];
	unsigned char srcAddr[6];
	unsigned short ethType;

	//total number of bytes in pkt, including this header
	unsigned short pktLength;
	
	//number of warpnet structs that follow
	unsigned char numStructs;
	
	//Sequence number to match packets to ACKs
	unsigned char seqNum;
} warpnetEthernetPktHeader;

//Struct containing one node's statistics
typedef struct {
	unsigned char structType;
	unsigned char nodeID;
	unsigned char partnerID;
	unsigned char reserved0;
	unsigned int goodPackets;
	unsigned int partnerBadPackets;
	unsigned int otherBadPackets;
	unsigned int rxBytes;
	unsigned int txBytes;
} warpnodeStats;

//Struct containing control parameters for the MAC/PHY
typedef struct {
	unsigned char structType;
	unsigned char nodeID;
	unsigned char modOrder;
	unsigned char txPower;
	unsigned char coding;
	unsigned char channel;
} warpnodeControl;

//Struct containing a command for a node
typedef struct {
	unsigned char structType;
	unsigned char nodeID;
	unsigned short cmdID;
	unsigned int cmdParam;
} warpnodeCommand;

//Struct containing just the node's ID (used to initialize node/server connection)
typedef struct {
	unsigned char structType;
	unsigned char nodeID;	
} warpnodeID;

//Struct Types for WARP Node <-> Server Packets
#define STRUCTID_NODESTATS 0x40
#define STRUCTID_NODEID 0x41
#define STRUCTID_NODECONTROL 0x42
#define STRUCTID_NODECOMMAND 0x43

//Node command codes
#define NODECMD_RESETSTATS 0x61
#define NODECMD_REBOOT 0x62
#define NODECMD_NODEACK 0x63

//Initial node identification stuff
#define NODEID_UNKNOWN 0xFF

#define WARPNET_ETHTYPE_SVR2NODE 0x9090
#define WARPNET_ETHTYPE_NODE2SVR 0x9191

#endif
